@@ -1,4 +1,4 @@
-/* Copyright (c) 2012-2021. The SimGrid Team.
+/* Copyright (c) 2012-2024. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -26,5 +26,5 @@ void free_message(void* message)
   const kademlia_message_t msg = (kademlia_message_t)message;
   if (msg)
     answer_free(msg->answer);
-  free(msg);
+  xbt_free(msg);
 }
