@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2021. The SimGrid Team.
+/* Copyright (c) 2007-2024. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -63,8 +63,8 @@ int main(int argc, char* argv[])
 
   simgrid_load_platform(argv[1]);
 
-  sg_actor_create("dvfs_test", sg_host_by_name("MyHost1"), dvfs, 0, NULL);
-  sg_actor_create("dvfs_test", sg_host_by_name("MyHost2"), dvfs, 0, NULL);
+  sg_actor_create("dvfs_test", sg_host_by_name("MyHost1"), &dvfs, 0, NULL);
+  sg_actor_create("dvfs_test", sg_host_by_name("MyHost2"), &dvfs, 0, NULL);
 
   simgrid_run();
 
