@@ -1,9 +1,10 @@
-/* Copyright (c) 2007-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include <simgrid/plugins/ProducerConsumer.hpp>
+#include <simgrid/s4u/ActivitySet.hpp>
 #include <simgrid/s4u/Actor.hpp>
 #include <simgrid/s4u/Engine.hpp>
 #include <simgrid/s4u/Host.hpp>
@@ -23,12 +24,14 @@ static void ingester(int id, simgrid::plugin::ProducerConsumerPtr<int> pc)
     sg4::this_actor::sleep_for((3 - i) * simgrid::xbt::random::uniform_real(0, 1));
   }
 
+  sg4::ActivitySet pending;
   for (int i = 0; i < 3; i++) {
     auto* data = new int(10 * id + i);
-    pc->put_async(data, 1.2125e6); // last for 0.01s
+    pending.push(pc->put_async(data, 1.2125e6)); // last for 0.01s
     XBT_INFO("data sucessfully put: %d", *data);
     sg4::this_actor::sleep_for((i + 3) * simgrid::xbt::random::uniform_real(0, 1));
   }
+  pending.wait_all();
 }
 
 static void retriever(simgrid::plugin::ProducerConsumerPtr<int> pc)
@@ -58,19 +61,16 @@ int main(int argc, char* argv[])
   // Platform creation
   auto* cluster = sg4::create_star_zone("cluster");
   for (int i = 0; i < 8; i++) {
-    std::string hostname = std::string("node-") + std::to_string(i) + ".simgrid.org";
+    std::string hostname = "node-" + std::to_string(i) + ".simgrid.org";
 
     const auto* host = cluster->create_host(hostname, "1Gf");
 
-    std::string linkname = std::string("cluster") + "_link_" + std::to_string(i);
+    std::string linkname = "cluster_link_" + std::to_string(i);
     const auto* link     = cluster->create_split_duplex_link(linkname, "1Gbps");
 
-    cluster->add_route(host->get_netpoint(), nullptr, nullptr, nullptr, {{link, sg4::LinkInRoute::Direction::UP}},
-                       true);
+    cluster->add_route(host, nullptr,  {{link, sg4::LinkInRoute::Direction::UP}}, true);
   }
-
-  auto* router = cluster->create_router("cluster_router");
-  cluster->add_route(router, nullptr, nullptr, nullptr, {});
+  cluster->seal();
 
   simgrid::plugin::ProducerConsumerPtr<int> pc = simgrid::plugin::ProducerConsumer<int>::create(2);
 
@@ -78,10 +78,10 @@ int main(int argc, char* argv[])
   XBT_INFO("Transfers are done in %s mode", pc->get_transfer_mode().c_str());
 
   for (int i = 0; i < 3; i++) {
-    std::string hostname = std::string("node-") + std::to_string(i) + ".simgrid.org";
+    std::string hostname = "node-" + std::to_string(i) + ".simgrid.org";
     sg4::Actor::create("ingester-" + std::to_string(i), e.host_by_name(hostname), &ingester, i, pc);
 
-    hostname = std::string("node-") + std::to_string(i + 3) + ".simgrid.org";
+    hostname = "node-" + std::to_string(i + 3) + ".simgrid.org";
     sg4::Actor::create("retriever-" + std::to_string(i), e.host_by_name(hostname), &retriever, pc);
   }
 
