@@ -1,16 +1,14 @@
-/* Copyright (c) 2018-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2018-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
+/* This file defines all possible exceptions that could occur in a SimGrid library. */
+
 #ifndef SIMGRID_EXCEPTIONS_HPP
 #define SIMGRID_EXCEPTIONS_HPP
 
-/** @file exception.hpp SimGrid-specific Exceptions
- *
- *  Defines all possible exception that could occur in a SimGrid library.
- */
-
+#include <simgrid/actor.h>
 #include <xbt/backtrace.hpp>
 #include <xbt/ex.h>
 #include <xbt/string.hpp>
@@ -28,15 +26,13 @@ namespace xbt {
  *  Constitute the contextual information of where an exception was thrown
  *
  *  These tuples (__FILE__, __LINE__, __func__, backtrace, procname, pid)
- *  are best created with @ref XBT_THROW_POINT.
- *
- *  @ingroup XBT_ex
+ *  are best created with the macro XBT_THROW_POINT.
  */
 class ThrowPoint {
 public:
   ThrowPoint() = default;
   explicit ThrowPoint(const char* file, int line, const char* function, Backtrace&& bt, std::string&& actor_name,
-                      int pid)
+                      aid_t pid)
       : file_(file)
       , line_(line)
       , function_(function)
@@ -51,12 +47,13 @@ public:
   const char* function_ = nullptr;
   Backtrace backtrace_;
   std::string procname_ = ""; /**< Name of the process who thrown this */
-  int pid_              = 0;  /**< PID of the process who thrown this */
+  aid_t pid_            = 0;  /**< PID of the process who thrown this */
 };
 
 /** Create a ThrowPoint with (__FILE__, __LINE__, __func__) */
 #define XBT_THROW_POINT                                                                                                \
-  ::simgrid::xbt::ThrowPoint(__FILE__, __LINE__, __func__, simgrid::xbt::Backtrace(), xbt_procname(), xbt_getpid())
+  ::simgrid::xbt::ThrowPoint(__FILE__, __LINE__, __func__, simgrid::xbt::Backtrace(), sg_actor_self_get_name(),        \
+                             sg_actor_self_get_pid())
 
 class XBT_PUBLIC ImpossibleError : public std::logic_error {
 public:
@@ -153,6 +150,9 @@ DECLARE_SIMGRID_EXCEPTION(TracingError);
 DECLARE_SIMGRID_EXCEPTION(ParseError, PARSE_ERROR_CONSTRUCTOR);
 #undef PARSE_ERROR_CONSTRUCTOR
 
+/** Exception raised by xbt_enforce, when an assertion is not satisfied */
+DECLARE_SIMGRID_EXCEPTION(AssertionError);
+
 #undef DECLARE_SIMGRID_EXCEPTION
 
 class XBT_PUBLIC ForcefulKillException {
@@ -180,17 +180,16 @@ class XBT_PUBLIC ForcefulKillException {
    */
 public:
   ForcefulKillException() = default;
-  explicit ForcefulKillException(const std::string& msg) : msg_(std::string("Actor killed (") + msg + std::string(")."))
-  {
-  }
+  explicit ForcefulKillException(const std::string& msg) : msg_("Actor killed (" + msg + ").") {}
   ~ForcefulKillException();
   const char* what() const noexcept { return msg_.c_str(); }
 
   XBT_ATTRIB_NORETURN static void do_throw();
-  static bool try_n_catch(const std::function<void()>& try_block);
+  XBT_ATTRIB_DEPRECATED_v338("Please manifest if you actually need this function") static bool try_n_catch(
+      const std::function<void()>& try_block);
 
 private:
-  std::string msg_ = std::string("Actor killed.");
+  std::string msg_ = "Actor killed.";
 };
 
 } // namespace simgrid
