@@ -1,4 +1,4 @@
-/* Copyright (c) 2018-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2018-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -11,9 +11,9 @@
 
 SG_BEGIN_DECL /* C interface */
 
-    /** Initialize the SimGrid engine, taking the command line parameters of your main function. */
-    XBT_PUBLIC void
-    simgrid_init(int* argc, char** argv);
+/** Initialize the SimGrid engine, taking the command line parameters of your main function. */
+XBT_PUBLIC void
+simgrid_init(int* argc, char** argv);
 
 /** Creates a new platform, including hosts, links, and the routing table.
  *
@@ -29,8 +29,10 @@ XBT_PUBLIC void simgrid_load_platform(const char* filename);
  * \endrst
  */
 XBT_PUBLIC void simgrid_load_deployment(const char* filename);
-/** Run the simulation after initialization */
+/** Run the simulation until its end */
 XBT_PUBLIC void simgrid_run();
+/** Run the simulation until the specified date */
+XBT_PUBLIC void simgrid_run_until(double max_date);
 /** Registers the main function of an actor that will be launched from the deployment file */
 XBT_PUBLIC void simgrid_register_function(const char* name, void (*code)(int, char**));
 /** Registers a function as the default main function of actors
@@ -41,20 +43,22 @@ XBT_PUBLIC void simgrid_register_function(const char* name, void (*code)(int, ch
 XBT_PUBLIC void simgrid_register_default(void (*code)(int, char**));
 /** Retrieve the simulation time (in seconds) */
 XBT_PUBLIC double simgrid_get_clock();
+/** PAPI */
+XBT_PUBLIC int simgrid_papi_get_num_counters();
+XBT_PUBLIC void simgrid_get_papi_counters( long long* );
+XBT_PUBLIC void simgrid_papi_start();
+XBT_PUBLIC void simgrid_papi_stop();
 /* Set some code to execute in the maestro (must be used before the engine creation)
  *
  * If no maestro code is registered (the default), the main thread
  * is assumed to be the maestro. */
 XBT_PUBLIC void simgrid_set_maestro(void (*code)(void*), void* data);
 
-/** Retrieve the number of actors in the simulation */
-XBT_ATTRIB_DEPRECATED_v330("Please use sg_actor_count()") XBT_PUBLIC int simgrid_get_actor_count();
-
 /** @brief Allow other libraries to react to the --help flag, too
  *
- * When finding --help on the command line, simgrid usually stops right after displaying its help message.
- * If you are writing a library using simgrid, you may want to display your own help message before everything stops.
- * If so, just call this function before having simgrid parsing the command line, and you will be given the control
+ * When finding --help on the command line, SimGrid usually stops right after displaying its help message.
+ * If you are writing a library using SimGrid, you may want to display your own help message before everything stops.
+ * If so, just call this function before having SimGrid parsing the command line, and you will be given the control
  * even if the user is asking for help.
  */
 XBT_PUBLIC void sg_config_continue_after_help();
