@@ -1,4 +1,4 @@
-/* Copyright (c) 2016-2021. The SimGrid Team. All rights reserved.               */
+/* Copyright (c) 2016-2024. The SimGrid Team. All rights reserved.               */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -9,6 +9,7 @@
 #include <simgrid/forward.h>
 #include <simgrid/s4u/Link.hpp>
 #include <simgrid/s4u/NetZone.hpp>
+#include <xbt/Extendable.hpp>
 #include <xbt/PropertyHolder.hpp>
 #include <xbt/graph.h>
 
@@ -17,8 +18,9 @@
 #include <vector>
 
 namespace simgrid {
-namespace kernel {
-namespace routing {
+extern template class XBT_PUBLIC xbt::Extendable<kernel::routing::NetZoneImpl>;
+
+namespace kernel::routing {
 
 class Route {
 public:
@@ -31,7 +33,7 @@ public:
   NetPoint* dst_    = nullptr;
   NetPoint* gw_src_ = nullptr;
   NetPoint* gw_dst_ = nullptr;
-  std::vector<resource::LinkImpl*> link_list_;
+  std::vector<resource::StandardLinkImpl*> link_list_;
 };
 
 class BypassRoute {
@@ -39,7 +41,7 @@ public:
   explicit BypassRoute(NetPoint* gwSrc, NetPoint* gwDst) : gw_src(gwSrc), gw_dst(gwDst) {}
   NetPoint* gw_src;
   NetPoint* gw_dst;
-  std::vector<resource::LinkImpl*> links;
+  std::vector<resource::StandardLinkImpl*> links;
 };
 
 /** @ingroup ROUTING_API
@@ -71,12 +73,18 @@ public:
  * called Autonomous Systems in this article).
  *
  */
-class XBT_PUBLIC NetZoneImpl : public xbt::PropertyHolder {
+class XBT_PUBLIC NetZoneImpl : public xbt::PropertyHolder, public xbt::Extendable<NetZoneImpl> {
   friend EngineImpl; // it destroys netRoot_
   s4u::NetZone piface_;
 
   // our content, as known to our graph routing algorithm (maps vertex_id -> vertex)
-  std::vector<kernel::routing::NetPoint*> vertices_;
+  std::vector<NetPoint*> vertices_;
+  std::map<std::string, resource::StandardLinkImpl*, std::less<>> links_;
+  /* save split-duplex links separately, keep links_ with only LinkImpl* seen by the user
+   * members of a split-duplex are saved in the links_ */
+  std::map<std::string, std::unique_ptr<resource::SplitDuplexLinkImpl>, std::less<>> split_duplex_links_;
+  std::map<std::string, resource::HostImpl*, std::less<>> hosts_;
+  std::map<std::string, NetPoint*, std::less<>> gateways_;
 
   NetZoneImpl* parent_ = nullptr;
   std::vector<NetZoneImpl*> children_; // sub-netzones
@@ -84,7 +92,7 @@ class XBT_PUBLIC NetZoneImpl : public xbt::PropertyHolder {
   bool sealed_ = false; // We cannot add more content when sealed
 
   std::map<std::pair<const NetPoint*, const NetPoint*>, BypassRoute*> bypass_routes_; // src x dst -> route
-  routing::NetPoint* netpoint_ = nullptr; // Our representative in the parent NetZone
+  NetPoint* netpoint_ = nullptr; // Our representative in the parent NetZone
 
 protected:
   explicit NetZoneImpl(const std::string& name);
@@ -104,15 +112,20 @@ protected:
   /** @brief retrieves the list of all routes of size 1 (of type src x dst x Link) */
   /* returns whether we found a bypass path */
   bool get_bypass_route(const routing::NetPoint* src, const routing::NetPoint* dst,
-                        /* OUT */ std::vector<resource::LinkImpl*>& links, double* latency,
+                        /* OUT */ std::vector<resource::StandardLinkImpl*>& links, double* latency,
                         std::unordered_set<NetZoneImpl*>& netzones);
 
   /** @brief Get the NetZone that is represented by the netpoint */
   const NetZoneImpl* get_netzone_recursive(const NetPoint* netpoint) const;
 
   /** @brief Get the list of LinkImpl* to add in a route, considering split-duplex links and the direction */
-  std::vector<resource::LinkImpl*> get_link_list_impl(const std::vector<s4u::LinkInRoute>& link_list,
-                                                      bool backroute) const;
+  std::vector<resource::StandardLinkImpl*> get_link_list_impl(const std::vector<s4u::LinkInRoute>& link_list,
+                                                              bool backroute) const;
+
+  static xbt_node_t new_xbt_graph_node(const s_xbt_graph_t* graph, const char* name,
+                                       std::map<std::string, xbt_node_t, std::less<>>* nodes);
+  static xbt_edge_t new_xbt_graph_edge(const s_xbt_graph_t* graph, xbt_node_t src, xbt_node_t dst,
+                                       std::map<std::string, xbt_edge_t, std::less<>>* edges);
 
 public:
   enum class RoutingMode {
@@ -129,13 +142,12 @@ public:
   /** @brief Retrieves the disk model associated to this NetZone */
   const std::shared_ptr<resource::DiskModel>& get_disk_model() const { return disk_model_; }
   /** @brief Retrieves the host model associated to this NetZone */
-  const std::shared_ptr<surf::HostModel>& get_host_model() const { return host_model_; }
+  const std::shared_ptr<resource::HostModel>& get_host_model() const { return host_model_; }
 
   const s4u::NetZone* get_iface() const { return &piface_; }
   s4u::NetZone* get_iface() { return &piface_; }
   unsigned int get_table_size() const { return vertices_.size(); }
-  std::vector<kernel::routing::NetPoint*> get_vertices() const { return vertices_; }
-  XBT_ATTRIB_DEPRECATED_v331("Please use get_parent()") NetZoneImpl* get_father() const { return parent_; }
+  std::vector<NetPoint*> get_vertices() const { return vertices_; }
   NetZoneImpl* get_parent() const { return parent_; }
   /** @brief Returns the list of direct children (no grand-children). This returns the internal data, no copy.
    * Don't mess with it.*/
@@ -149,17 +161,78 @@ public:
   const char* get_cname() const { return name_.c_str(); };
 
   /** @brief Gets the netpoint associated to this netzone */
-  kernel::routing::NetPoint* get_netpoint() const { return netpoint_; }
+  NetPoint* get_netpoint() const { return netpoint_; }
+
+  void set_gateway(const std::string& name, NetPoint* router);
+  /** @brief Gets the gateway associated to this netzone */
+  NetPoint* get_gateway() const;
+  NetPoint* get_gateway(const std::string& name) const { return gateways_.at(name); }
 
   std::vector<s4u::Host*> get_all_hosts() const;
-  int get_host_count() const;
+  size_t get_host_count() const;
+
+  /**
+   * @brief Recursively gets all links declared in this netzone
+   *
+   * Include children netzones.
+   * @return List of links
+   */
+  std::vector<s4u::Link*> get_all_links() const;
+  /**
+   * @brief Recursively gets all links declared in this netzone.
+   *
+   * Using a filter function
+   * Include children netzones.
+   * @param filter Select links based on this filter
+   * @return List of links
+   */
+  std::vector<s4u::Link*> get_filtered_links(const std::function<bool(s4u::Link*)>& filter) const;
+  /** @brief Get total number of links declared in this netzone (and its children) */
+  size_t get_link_count() const;
+
+  /**
+   * @brief Searches by the link by its name inside this netzone.
+   * Recursively searches in children netzones
+   *
+   * @param name Link name
+   * @return Link object or nullptr if not found
+   */
+  resource::StandardLinkImpl* get_link_by_name_or_null(const std::string& name) const;
+
+  /**
+   * @brief Searches for split-duplex links by its name inside this netzone.
+   * Recursively searches in child netzones
+   *
+   * @param name Split-duplex Link name
+   * @return Link object or nullptr if not found
+   */
+  resource::SplitDuplexLinkImpl* get_split_duplex_link_by_name_or_null(const std::string& name) const;
+
+  /**
+   * @brief Searches for a host by its name (recursively)
+   * Including children netzones and VMs on physival hosts
+   *
+   * @param name Host (or VM) name
+   * @return HostImpl pointer
+   */
+  resource::HostImpl* get_host_by_name_or_null(const std::string& name) const;
+
+  /**
+   * @brief Gets list of hosts on this netzone recursively.
+   *
+   * Note: This includes hosts on children netzones and VMs on physical hosts.
+   *
+   * @param filter Filter function to select specific nodes
+   * @return List of hosts
+   */
+  std::vector<s4u::Host*> get_filtered_hosts(const std::function<bool(s4u::Host*)>& filter) const;
 
   /** @brief Make a host within that NetZone */
   s4u::Host* create_host(const std::string& name, const std::vector<double>& speed_per_pstate);
   /** @brief Create a disk with the disk model from this NetZone */
   s4u::Disk* create_disk(const std::string& name, double read_bandwidth, double write_bandwidth);
   /** @brief Make a link within that NetZone */
-  virtual s4u::Link* create_link(const std::string& name, const std::vector<double>& bandwidths);
+  s4u::Link* create_link(const std::string& name, const std::vector<double>& bandwidths);
   s4u::SplitDuplexLink* create_split_duplex_link(const std::string& name, const std::vector<double>& bandwidths);
   /** @brief Make a router within that NetZone */
   NetPoint* create_router(const std::string& name);
@@ -181,7 +254,7 @@ public:
   void set_cpu_vm_model(std::shared_ptr<resource::CpuModel> cpu_model);
   void set_cpu_pm_model(std::shared_ptr<resource::CpuModel> cpu_model);
   void set_disk_model(std::shared_ptr<resource::DiskModel> disk_model);
-  void set_host_model(std::shared_ptr<surf::HostModel> host_model);
+  void set_host_model(std::shared_ptr<resource::HostModel> host_model);
 
   /** @brief get the route between two nodes in the full platform
    *
@@ -191,20 +264,20 @@ public:
    * @param latency Accumulator in which the latencies should be added (caller must set it to 0)
    */
   static void get_global_route(const NetPoint* src, const NetPoint* dst,
-                               /* OUT */ std::vector<resource::LinkImpl*>& links, double* latency);
+                               /* OUT */ std::vector<resource::StandardLinkImpl*>& links, double* latency);
 
   /** @brief Similar to get_global_route but get the NetZones traversed by route */
   static void get_global_route_with_netzones(const NetPoint* src, const NetPoint* dst,
-                                             /* OUT */ std::vector<resource::LinkImpl*>& links, double* latency,
+                                             /* OUT */ std::vector<resource::StandardLinkImpl*>& links, double* latency,
                                              std::unordered_set<NetZoneImpl*>& netzones);
 
   virtual void get_graph(const s_xbt_graph_t* graph, std::map<std::string, xbt_node_t, std::less<>>* nodes,
-                         std::map<std::string, xbt_edge_t, std::less<>>* edges) = 0;
+                         std::map<std::string, xbt_edge_t, std::less<>>* edges);
 
   /*** Called on each newly created regular route (not on bypass routes) */
   static xbt::signal<void(bool symmetrical, NetPoint* src, NetPoint* dst, NetPoint* gw_src, NetPoint* gw_dst,
-                          std::vector<resource::LinkImpl*> const& link_list)>
-      on_route_creation; // XBT_ATTRIB_DEPRECATED_v332 : should be an internal signal used by NS3.. if necessary,
+                          std::vector<resource::StandardLinkImpl*> const& link_list)>
+      on_route_creation; // FIXME: XBT_ATTRIB_DEPRECATED_v332: should be an internal signal used by NS3.. if necessary,
                          // callback shouldn't use LinkImpl*
 
 private:
@@ -213,15 +286,17 @@ private:
   std::shared_ptr<resource::CpuModel> cpu_model_vm_;
   std::shared_ptr<resource::CpuModel> cpu_model_pm_;
   std::shared_ptr<resource::DiskModel> disk_model_;
-  std::shared_ptr<surf::HostModel> host_model_;
+  std::shared_ptr<resource::HostModel> host_model_;
   /** @brief Perform sealing procedure for derived classes, if necessary */
   virtual void do_seal()
   { /* obviously nothing to do by default */
   }
+  /** @brief Allows subclasses (wi-fi) to have their own create link method, but keep links_ updated */
+  virtual resource::StandardLinkImpl* do_create_link(const std::string& name, const std::vector<double>& bandwidths);
   void add_child(NetZoneImpl* new_zone);
 };
-} // namespace routing
-} // namespace kernel
+
+} // namespace kernel::routing
 } // namespace simgrid
 
 #endif /* SIMGRID_ROUTING_NETZONEIMPL_HPP */
