@@ -1,4 +1,4 @@
-/* Copyright (c) 2016-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2016-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -31,18 +31,6 @@ XBT_PUBLIC int sg_link_energy_is_inited();
  *  @brief Initialize the wifi energy plugin */
 XBT_PUBLIC void sg_wifi_energy_plugin_init();
 
-#if SIMGRID_HAVE_MSG
-
-#define MSG_host_energy_plugin_init() sg_host_energy_plugin_init()
-#define MSG_host_get_consumed_energy(host) sg_host_get_consumed_energy(host)
-#define MSG_host_get_idle_consumption_at(host,pstate) sg_host_get_idle_consumption_at((host), (pstate))
-#define MSG_host_get_wattmin_at(host,pstate) sg_host_get_wattmin_at((host), (pstate))
-#define MSG_host_get_wattmax_at(host,pstate) sg_host_get_wattmax_at((host), (pstate))
-#define MSG_host_get_power_range_slope_at(host,pstate) sg_host_get_power_range_slope_at((host), (pstate))
-#define MSG_host_get_current_consumption(host) sg_host_get_current_consumption(host)
-
-#endif // SIMGRID_HAVE_MSG
-
 SG_END_DECL
 
 #endif
