@@ -1,4 +1,4 @@
-/* Copyright (c) 2018-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2018-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -15,31 +15,26 @@
 #include <atomic>
 #include <future>
 
-namespace simgrid {
-namespace s4u {
+namespace simgrid::s4u {
 
 class XBT_PUBLIC Barrier {
-private:
-  MutexPtr mutex_            = Mutex::create();
-  ConditionVariablePtr cond_ = ConditionVariable::create();
-  unsigned int expected_actors_;
-  unsigned int arrived_actors_ = 0;
+  kernel::activity::BarrierImpl* pimpl_;
+  friend kernel::activity::BarrierImpl;
 
-  /* refcounting */
-  std::atomic_int_fast32_t refcount_{0};
+  explicit Barrier(kernel::activity::BarrierImpl* pimpl) : pimpl_(pimpl) {}
 
 public:
-  /** Creates a barrier for the given amount of actors */
-  explicit Barrier(unsigned int expected_actors) : expected_actors_(expected_actors) {}
 #ifndef DOXYGEN
   Barrier(Barrier const&) = delete;
   Barrier& operator=(Barrier const&) = delete;
 #endif
 
-  /** Creates a barrier for the given amount of actors */
+  /** \static Creates a barrier for the given amount of actors */
   static BarrierPtr create(unsigned int expected_actors);
   /** Blocks into the barrier. Every waiting actors will be unlocked once the expected amount of actors reaches the barrier */
   int wait();
+  /** Returns some debug information about the barrier */
+  std::string to_string() const;
 
 #ifndef DOXYGEN
   /* refcounting */
@@ -47,7 +42,6 @@ public:
   friend XBT_PUBLIC void intrusive_ptr_release(Barrier* barrier);
 #endif
 };
-} // namespace s4u
-} // namespace simgrid
+} // namespace simgrid::s4u
 
 #endif
