@@ -1,4 +1,4 @@
-/* Copyright (c) 2017-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2017-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -11,8 +11,7 @@
 #include <simgrid/s4u/Actor.hpp>
 #include <xbt/ex.h>
 
-namespace simgrid {
-namespace s4u {
+namespace simgrid::s4u {
 
 /** Computation Activity, representing the asynchronous executions.
  *
@@ -32,63 +31,78 @@ namespace s4u {
 class XBT_PUBLIC Exec : public Activity_T<Exec> {
 #ifndef DOXYGEN
   friend kernel::activity::ExecImpl;
+  friend kernel::EngineImpl; // Auto-completes the execs of maestro (in simDAG)
 #endif
 
   bool parallel_ = false;
 
 protected:
   explicit Exec(kernel::activity::ExecImplPtr pimpl);
+  Exec* do_start() override;
 
-  void complete(Activity::State state) override;
+  void reset() const;
 
 public:
 #ifndef DOXYGEN
   Exec(Exec const&) = delete;
   Exec& operator=(Exec const&) = delete;
 #endif
-  static xbt::signal<void(Exec const&)> on_start;
-  static xbt::signal<void(Exec const&)> on_completion;
-
+  /*! \static Initiate the creation of an Exec. Setters have to be called afterwards */
   static ExecPtr init();
-  Exec* start() override;
-
-  /*! take a vector of s4u::ExecPtr and return when one of them is finished.
-   * The return value is the rank of the first finished ExecPtr. */
-  static ssize_t wait_any(const std::vector<ExecPtr>& execs) { return wait_any_for(execs, -1); }
-  /*! Same as wait_any, but with a timeout. If the timeout occurs, parameter last is returned.*/
-  static ssize_t wait_any_for(const std::vector<ExecPtr>& execs, double timeout);
-
-#ifndef DOXYGEN
-  XBT_ATTRIB_DEPRECATED_v332("Please use a plain vector for parameter")
-  static int wait_any(std::vector<ExecPtr>* execs) { return static_cast<int>(wait_any_for(*execs, -1)); }
-  XBT_ATTRIB_DEPRECATED_v332("Please use a plain vector for first parameter")
-  static int wait_any_for(std::vector<ExecPtr>* execs, double timeout) { return static_cast<int>(wait_any_for(*execs, timeout)); }
-#endif
 
   /** @brief On sequential executions, returns the amount of flops that remain to be done; This cannot be used on
    * parallel executions. */
   double get_remaining() const override;
+  /** @brief Returns the ratio of elements that are still to do
+   *
+   * The returned value is between 0 (completely done) and 1 (nothing done yet). */
   double get_remaining_ratio() const;
+  /** @brief Change the host on which this activity takes place.
+   *
+   * This cannot be done once the activity is terminated, but it can be done on started executions. */
   ExecPtr set_host(Host* host);
   ExecPtr set_hosts(const std::vector<Host*>& hosts);
+  ExecPtr unset_host();
+  ExecPtr unset_hosts() { return unset_host(); }
 
   ExecPtr set_flops_amount(double flops_amount);
   ExecPtr set_flops_amounts(const std::vector<double>& flops_amounts);
   ExecPtr set_bytes_amounts(const std::vector<double>& bytes_amounts);
 
-  ExecPtr set_bound(double bound);
-  ExecPtr set_priority(double priority);
+  ExecPtr set_thread_count(int thread_count);
 
+  /** @brief change the execution bound
+   * This means changing the maximal amount of flops per second that it may consume, regardless of what the host may
+   * deliver. Currently, this cannot be changed once the exec started. See also the "cloud-capping" example.  */
+  ExecPtr set_bound(double bound);
+
+  /** @brief  Change the execution priority, don't you think?
+   * An execution with twice the priority will get twice the amount of flops when the resource is shared.
+   * The default priority is 1.
+   *
+   * Currently, this cannot be changed once the exec started. */
+  ExecPtr set_priority(double priority);
+  ExecPtr update_priority(double priority);
+
+  /** @brief Retrieve the host on which this activity takes place.
+   *  If it runs on more than one host, only the first host is returned. */
   Host* get_host() const;
   unsigned int get_host_number() const;
-  double get_start_time() const;
-  double get_finish_time() const;
+  int get_thread_count() const;
   double get_cost() const;
   bool is_parallel() const { return parallel_; }
   bool is_assigned() const override;
+
+#ifndef DOXYGEN
+  static ssize_t deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double timeout); // XBT_ATTRIB_DEPRECATED_v339
+
+  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t
+      wait_any(const std::vector<ExecPtr>& execs) { return deprecated_wait_any_for(execs, -1); }
+  XBT_ATTRIB_DEPRECATED_v339("Please use ActivitySet instead") static ssize_t
+      wait_any_for(const std::vector<ExecPtr>& execs, double timeout) { return deprecated_wait_any_for(execs, timeout); }
+#endif
 };
 
-} // namespace s4u
-} // namespace simgrid
+} // namespace simgrid::s4u
 
 #endif /* SIMGRID_S4U_EXEC_HPP */
