@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -7,9 +7,9 @@
 #define SIMGRID_S4U_HOST_HPP
 
 #include <simgrid/forward.h>
+#include <simgrid/kernel/resource/Action.hpp>
 #include <xbt/Extendable.hpp>
 #include <xbt/signal.hpp>
-#include <xbt/string.hpp>
 
 #include <map>
 #include <unordered_map>
@@ -37,35 +37,76 @@ namespace s4u {
  */
 class XBT_PUBLIC Host : public xbt::Extendable<Host> {
 #ifndef DOXYGEN
-  friend vm::VMModel;            // Use the pimpl_cpu to compute the VM sharing
-  friend vm::VirtualMachineImpl; // creates the the pimpl_cpu
+  friend kernel::resource::VMModel;            // Use the pimpl_cpu to compute the VM sharing
+  friend kernel::resource::VirtualMachineImpl; // creates the the pimpl_cpu
   friend kernel::routing::NetZoneImpl;
-  friend surf::HostImpl; // call destructor from private implementation
+  friend kernel::resource::HostImpl; // call destructor from private implementation
+  friend kernel::resource::CpuAction; // signal exec_state_changed
 
   // The private implementation, that never changes
-  surf::HostImpl* const pimpl_;
+  kernel::resource::HostImpl* const pimpl_;
 
   kernel::resource::CpuImpl* pimpl_cpu_      = nullptr;
   kernel::routing::NetPoint* pimpl_netpoint_ = nullptr;
 
+  static xbt::signal<void(Host&)> on_creation;
+  static xbt::signal<void(Host const&)> on_destruction;
+  xbt::signal<void(Host const&)> on_this_destruction;
+  static xbt::signal<void(kernel::resource::CpuAction&, kernel::resource::Action::State)> on_exec_state_change;
+
 public:
-  explicit Host(surf::HostImpl* pimpl) : pimpl_(pimpl) {}
+  explicit Host(kernel::resource::HostImpl* pimpl) : pimpl_(pimpl) {}
 
 protected:
   virtual ~Host(); // Call destroy() instead of manually deleting it.
   Host* set_netpoint(kernel::routing::NetPoint* netpoint);
-#endif
 
 public:
-  /** Called on each newly created host */
-  static xbt::signal<void(Host&)> on_creation;
-  /** Called when the machine is turned on or off (called AFTER the change) */
-  static xbt::signal<void(Host const&)> on_state_change;
-  /** Called when the speed of the machine is changed (called AFTER the change)
-   * (either because of a pstate switch or because of an external load event coming from the profile) */
   static xbt::signal<void(Host const&)> on_speed_change;
-  /** Called just before destructing a host */
-  static xbt::signal<void(Host const&)> on_destruction;
+  xbt::signal<void(Host const&)> on_this_speed_change;
+  static xbt::signal<void(Host const&)> on_onoff;
+  xbt::signal<void(Host const&)> on_this_onoff;
+
+#endif
+  /** \static Add a callback fired on each newly created host */
+  static void on_creation_cb(const std::function<void(Host&)>& cb) { on_creation.connect(cb); }
+  /** \static Add a callback fired when any machine is turned on or off (called AFTER the change) */
+  static void on_onoff_cb(const std::function<void(Host const&)>& cb)
+  {
+    on_onoff.connect(cb);
+  }
+  XBT_ATTRIB_DEPRECATED_v338("Please use on_onoff_cb() instead") static void on_state_change_cb(
+      const std::function<void(Host const&)>& cb)
+  {
+    on_onoff.connect(cb);
+  }
+  /** Add a callback fired when this specific machine is turned on or off (called AFTER the change) */
+  void on_this_onoff_cb(const std::function<void(Host const&)>& cb)
+  {
+    on_this_onoff.connect(cb);
+  }
+  /** \static Add a callback fired when the speed of any machine is changed (called AFTER the change)
+   * (either because of a pstate switch or because of an external load event coming from the profile) */
+  static void on_speed_change_cb(const std::function<void(Host const&)>& cb) { on_speed_change.connect(cb); }
+  /** Add a callback fired when the speed of this specific machine is changed (called AFTER the change)
+   * (either because of a pstate switch or because of an external load event coming from the profile) */
+  void on_this_speed_change_cb(const std::function<void(Host const&)>& cb)
+  {
+    on_this_speed_change.connect(cb);
+  }
+  /** \static Add a callback fired just before destructing any host */
+  static void on_destruction_cb(const std::function<void(Host const&)>& cb) { on_destruction.connect(cb); }
+  /** Add a callback fired just before destructing this specific host */
+  void on_this_destruction_cb(const std::function<void(Host const&)>& cb)
+  {
+    on_this_destruction.connect(cb);
+  }
+  /** \static Add a callback fired when the state of any exec activity changes */
+  static void on_exec_state_change_cb(
+      const std::function<void(kernel::resource::CpuAction&, kernel::resource::Action::State previous)>& cb)
+  {
+    on_exec_state_change.connect(cb);
+  }
 
   virtual void destroy();
 #ifndef DOXYGEN
@@ -74,15 +115,15 @@ public:
   Host& operator=(Host const&) = delete;
 #endif
 
-  /** Retrieve a host from its name, or return nullptr */
+  /** \static Retrieve a host from its name, or return nullptr */
   static Host* by_name_or_null(const std::string& name);
-  /** Retrieve a host from its name, or die */
+  /** \static Retrieve a host from its name, or die */
   static Host* by_name(const std::string& name);
-  /** Retrieves the host on which the running actor is located */
+  /** \static Retrieves the host on which the running actor is located */
   static Host* current();
 
   /** Retrieves the name of that host as a C++ string */
-  xbt::string const& get_name() const;
+  std::string const& get_name() const;
   /** Retrieves the name of that host as a C string */
   const char* get_cname() const;
 
@@ -129,7 +170,15 @@ public:
   Host* set_state_profile(kernel::profile::Profile* p);
   Host* set_speed_profile(kernel::profile::Profile* p);
 
-  /** @brief Convert the CPU's speed from string to double */
+  /**
+   * @brief Set the max amount of executions that can take place on this host at the same time
+   *
+   * Use -1 to set no limit.
+   */
+  Host* set_concurrency_limit(int limit);
+  int get_concurrency_limit() const;
+
+  /** \static @brief Convert the CPU's speed from string to double */
   static std::vector<double> convert_pstate_speed_vector(const std::vector<std::string>& speed_per_state);
   /**
    * @brief Set the CPU's speed
@@ -147,13 +196,19 @@ public:
   /** @brief Get the peak computing speed in flops/s at the current pstate, NOT taking the external load into account.
    *
    *  The amount of flops per second available for computing depends on several things:
-   *    - The current pstate determines the maximal peak computing speed (use @ref get_pstate_speed() to retrieve the
-   *      computing speed you would get at another pstate)
-   *    - If you declared an external load (with @ref set_speed_profile()), you must multiply the
-   * result of get_speed() by get_available_speed() to retrieve what a new computation would get.
+   *    - The current pstate determines the maximal peak computing speed (use
+   *      @verbatim embed:rst:inline :cpp:func:`get_pstate_speed() <simgrid::s4u::Host::get_pstate_speed>` @endverbatim
+   *      to retrieve the computing speed you would get at another pstate)
+   *    - If you declared an external load (with
+   *      @verbatim embed:rst:inline :cpp:func:`set_speed_profile() <simgrid::s4u::Host::set_speed_profile>` @endverbatim ),
+   *      you must multiply the result of
+   *      @verbatim embed:rst:inline :cpp:func:`get_speed() <simgrid::s4u::Host::get_speed>` @endverbatim by
+   *      @verbatim embed:rst:inline :cpp:func:`get_available_speed() <simgrid::s4u::Host::get_available_speed>` @endverbatim
+   *      to retrieve what a new computation would get.
    *
    *  The remaining speed is then shared between the executions located on this host.
-   *  You can retrieve the amount of tasks currently running on this host with @ref get_load().
+   *  You can retrieve the amount of tasks currently running on this host with
+   *  @verbatim embed:rst:inline :cpp:func:`get_load() <simgrid::s4u::Host::get_load>` @endverbatim .
    *
    *  The host may have multiple cores, and your executions may be able to use more than a single core.
    *
@@ -162,7 +217,8 @@ public:
   double get_speed() const;
   /** @brief Get the available speed ratio, between 0 and 1.
    *
-   * This accounts for external load (see @ref set_speed_profile()).
+   * This accounts for external load (see
+   * @verbatim embed:rst:inline :cpp:func:`set_speed_profile() <simgrid::s4u::Host::set_speed_profile>` @endverbatim ).
    */
   double get_available_speed() const;
 
@@ -196,6 +252,8 @@ public:
   Host* set_coordinates(const std::string& coords);
 
   std::vector<Disk*> get_disks() const;
+  Disk* get_disk_by_name(const std::string& name) const;
+
   /**
    * @brief Create and add disk in the host
    *
@@ -213,8 +271,14 @@ public:
   void add_disk(const Disk* disk);
   void remove_disk(const std::string& disk_name);
 
+  VirtualMachine* create_vm(const std::string& name, int core_amount);
+  VirtualMachine* create_vm(const std::string& name, int core_amount, size_t ramsize);
+  /** Retrieve a VM running on this host from its name, or return nullptr */
+  VirtualMachine* vm_by_name_or_null(const std::string& name);
+
   void route_to(const Host* dest, std::vector<Link*>& links, double* latency) const;
-  void route_to(const Host* dest, std::vector<kernel::resource::LinkImpl*>& links, double* latency) const;
+  void route_to(const Host* dest, std::vector<kernel::resource::StandardLinkImpl*>& links, double* latency) const;
+  std::pair<std::vector<Link*>, double> route_to(const Host* dest) const;
 
   /**
    * @brief Seal this host
@@ -222,12 +286,6 @@ public:
    */
   Host* seal();
 
-#ifndef DOXYGEN
-  XBT_ATTRIB_DEPRECATED_v331("Please use Comm::sendto()") void sendto(Host* dest, double byte_amount);
-  XBT_ATTRIB_DEPRECATED_v331("Please use Comm::sendto_async()") CommPtr sendto_async(Host* dest, double byte_amount);
-  XBT_ATTRIB_DEPRECATED_v330("Please use Host::sendto()") void send_to(Host* dest, double byte_amount);
-#endif
-
   NetZone* get_englobing_zone() const;
   /** Block the calling actor on an execution located on the called host
    *
@@ -241,7 +299,7 @@ public:
 
   /** Block the calling actor on an execution located on the called host (with explicit priority) */
   void execute(double flops, double priority) const;
-  surf::HostImpl* get_impl() const { return pimpl_; }
+  kernel::resource::HostImpl* get_impl() const { return pimpl_; }
 };
 } // namespace s4u
 } // namespace simgrid
