@@ -1,4 +1,4 @@
-/* Copyright (c) 2004-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2004-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -31,22 +31,39 @@ namespace s4u {
  */
 class XBT_PUBLIC Link : public xbt::Extendable<Link> {
 #ifndef DOXYGEN
-  friend kernel::resource::LinkImpl;
+  friend kernel::resource::StandardLinkImpl;
 #endif
 
 protected:
   // Links are created from the NetZone, and destroyed by their private implementation when the simulation ends
-  explicit Link(kernel::resource::LinkImplIntf* pimpl) : pimpl_(pimpl) {}
+  explicit Link(kernel::resource::LinkImpl* pimpl) : pimpl_(pimpl) {}
   virtual ~Link() = default;
   // The implementation that never changes
-  kernel::resource::LinkImplIntf* const pimpl_;
+  kernel::resource::LinkImpl* const pimpl_;
+#ifndef DOXYGEN
+  friend kernel::resource::NetworkAction; // signal comm_state_changed
+#endif
 
 public:
-  enum class SharingPolicy { NONLINEAR = 4, WIFI = 3, SPLITDUPLEX = 2, SHARED = 1, FATPIPE = 0 };
+  /** Specifies how a given link is shared between concurrent communications */
+  enum class SharingPolicy {
+    /// This policy takes a callback that specifies the maximal capacity as a function of the number of usage. See the
+    /// examples with 'degradation' in their name.
+    NONLINEAR = 4,
+    /// Pseudo-sharing policy requesting wifi-specific sharing.
+    WIFI = 3,
+    /// Each link is split in 2, UP and DOWN, one per direction. These links are SHARED.
+    SPLITDUPLEX = 2,
+    /// The bandwidth is shared between all comms using that link, regardless of their direction.
+    SHARED = 1,
+    /// Each comm can use the link fully, with no sharing (only a maximum). This is intended to represent the backbone
+    /// links that cannot be saturated by concurrent links, but have a maximal bandwidth.
+    FATPIPE = 0
+  };
 
-  kernel::resource::LinkImpl* get_impl() const;
+  kernel::resource::StandardLinkImpl* get_impl() const;
 
-  /** @brief Retrieve a link from its name */
+  /** \static @brief Retrieve a link from its name */
   static Link* by_name(const std::string& name);
   static Link* by_name_or_null(const std::string& name);
 
@@ -108,6 +125,7 @@ public:
    * @param limit  Number of concurrent flows
    */
   Link* set_concurrency_limit(int limit);
+  int get_concurrency_limit() const;
 
   /** @brief Set the level of communication speed of the given host on this wifi link.
    *
@@ -119,11 +137,18 @@ public:
    * current function to specify that a given host uses another level of bandwidth. This can be used to take the
    * location of hosts into account, or even to model mobility in your SimGrid simulation.
    *
-   * Note that this function asserts that the link is actually a wifi link */
+   * Note that this function asserts that the link is actually a wifi link
+   *
+   * warning: in the case where a 0Mbps data rate should be used, set that rate only once during the
+   * experiment, and don't modify the bandwidth of that host later */
   void set_host_wifi_rate(const s4u::Host* host, int level) const;
 
   /** @brief Returns the current load (in bytes per second) */
-  double get_usage() const;
+  double get_load() const;
+
+#ifndef DOXYGEN
+  XBT_ATTRIB_DEPRECATED_v338("Please use get_load() instead") double get_usage() const { return get_load(); }
+#endif
 
   /** @brief Check if the Link is used (at least one flow uses the link) */
   bool is_used() const;
@@ -131,31 +156,68 @@ public:
   /** @brief Check if the Link is shared (not a FATPIPE) */
   bool is_shared() const;
 
+  /** Turns the link on. */
   void turn_on();
+  /** Turns the link off. */
   void turn_off();
+  /** Checks whether the link is on. */
   bool is_on() const;
 
   Link* seal();
 
-  /* The signals */
-  /** @brief Callback signal fired when a new Link is created */
+private:
+#ifndef DOXYGEN
   static xbt::signal<void(Link&)> on_creation;
-
-  /** @brief Callback signal fired when the state of a Link changes (when it is turned on or off) */
-  static xbt::signal<void(Link const&)> on_state_change;
-
-  /** @brief Callback signal fired when the bandwidth of a Link changes */
+  static xbt::signal<void(Link const&)> on_onoff;
+  xbt::signal<void(Link const&)> on_this_onoff;
   static xbt::signal<void(Link const&)> on_bandwidth_change;
-
-  /** @brief Callback signal fired when a communication starts */
-  static xbt::signal<void(kernel::resource::NetworkAction&)> on_communicate;
-
-  /** @brief Callback signal fired when a communication changes it state (ready/done/cancel) */
+  xbt::signal<void(Link const&)> on_this_bandwidth_change;
   static xbt::signal<void(kernel::resource::NetworkAction&, kernel::resource::Action::State)>
       on_communication_state_change;
-
-  /** @brief Callback signal fired when a Link is destroyed */
   static xbt::signal<void(Link const&)> on_destruction;
+  xbt::signal<void(Link const&)> on_this_destruction;
+#endif
+
+public:
+  /* The signals */
+  /** \static @brief Add a callback fired when a new Link is created */
+  static void on_creation_cb(const std::function<void(Link&)>& cb) { on_creation.connect(cb); }
+  /** \static @brief Add a callback fired when any Link is turned on or off */
+  static void on_onoff_cb(const std::function<void(Link const&)>& cb)
+  {
+    on_onoff.connect(cb);
+  }
+  /** @brief Add a callback fired when this specific Link is turned on or off */
+  void on_this_onoff_cb(const std::function<void(Link const&)>& cb)
+  {
+    on_this_onoff.connect(cb);
+  }
+  /** \static @brief Add a callback fired when the bandwidth of any Link changes */
+  static void on_bandwidth_change_cb(const std::function<void(Link const&)>& cb) { on_bandwidth_change.connect(cb); }
+  /** @brief Add a callback fired when the bandwidth of this specific Link changes */
+  void on_this_bandwidth_change_cb(const std::function<void(Link const&)>& cb)
+  {
+    on_this_bandwidth_change.connect(cb);
+  }
+  /** \static @brief Add a callback fired when a communication changes it state (ready/done/cancel) */
+  static void on_communication_state_change_cb(
+      const std::function<void(kernel::resource::NetworkAction&, kernel::resource::Action::State)>& cb)
+  {
+    on_communication_state_change.connect(cb);
+  }
+  /** \static @brief Add a callback fired when any Link is destroyed */
+  static void on_destruction_cb(const std::function<void(Link const&)>& cb) { on_destruction.connect(cb); }
+  /** @brief Add a callback fired when this specific Link is destroyed */
+  void on_this_destruction_cb(const std::function<void(Link const&)>& cb)
+  {
+    on_this_destruction.connect(cb);
+  }
+
+  XBT_ATTRIB_DEPRECATED_v338("Please use on_onoff_cb() instead") static void on_state_change_cb(
+      const std::function<void(Link const&)>& cb)
+  {
+    on_onoff.connect(cb);
+  }
 };
 
 /**
@@ -167,13 +229,13 @@ public:
  */
 class XBT_PUBLIC SplitDuplexLink : public Link {
 public:
-  explicit SplitDuplexLink(kernel::resource::LinkImplIntf* pimpl) : Link(pimpl) {}
+  explicit SplitDuplexLink(kernel::resource::LinkImpl* pimpl) : Link(pimpl) {}
   /** @brief Get the link direction up*/
   Link* get_link_up() const;
   /** @brief Get the link direction down */
   Link* get_link_down() const;
 
-  /** @brief Retrieve a link from its name */
+  /** \static @brief Retrieve a link from its name */
   static SplitDuplexLink* by_name(const std::string& name);
 };
 
