@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -7,16 +7,16 @@
 #define SIMGRID_S4U_MAILBOX_HPP
 
 #include <simgrid/forward.h>
+
+#include <simgrid/Exception.hpp>
 #include <simgrid/s4u/Actor.hpp>
 #include <simgrid/s4u/Comm.hpp>
 #include <smpi/forward.hpp>
-#include <xbt/string.hpp>
 
 #include <memory>
 #include <string>
 
-namespace simgrid {
-namespace s4u {
+namespace simgrid::s4u {
 
 /** @brief Mailboxes: Network rendez-vous points. */
 class XBT_PUBLIC Mailbox {
@@ -36,11 +36,11 @@ protected:
 
 public:
   /** @brief Retrieves the name of that mailbox as a C++ string */
-  const xbt::string& get_name() const;
+  const std::string& get_name() const;
   /** @brief Retrieves the name of that mailbox as a C string */
   const char* get_cname() const;
 
-  /** Retrieve the mailbox associated to the given name. Mailboxes are created on demand. */
+  /** \static Retrieve the mailbox associated to the given name. Mailboxes are created on demand. */
   static Mailbox* by_name(const std::string& name);
 
   /** Returns whether the mailbox contains queued communications */
@@ -55,7 +55,11 @@ public:
   /** Look if there is a communication going on in a mailbox and return the PID of the sender actor */
   aid_t listen_from() const;
 
-  /** Check if there is a communication ready to be consumed from a mailbox. */
+  /** Check if there is a communication ready to be consumed from a mailbox.
+   * \beginrst
+   *  See :ref:`this example <s4u_ex_mailbox_ready>`.
+   * \endrst
+   */
   bool ready() const;
 
   /** Gets the first element in the queue (without dequeuing it), or nullptr if none is there */
@@ -99,8 +103,9 @@ public:
    */
   CommPtr put_async(void* data, uint64_t simulated_size_in_bytes);
 
-  kernel::activity::ActivityImplPtr iprobe(int type, bool (*match_fun)(void*, void*, kernel::activity::CommImpl*),
-                                           void* data);
+  XBT_DECLARE_ENUM_CLASS(IprobeKind, SEND, RECV);
+  kernel::activity::ActivityImplPtr
+  iprobe(IprobeKind kind, const std::function<bool(void*, void*, kernel::activity::CommImpl*)>& match_fun, void* data);
   /** Blocking data transmission.
    *
    * Please note that if you send a pointer to some data, you must ensure that your data remains live during the
@@ -110,10 +115,15 @@ public:
   /** Blocking data transmission with timeout */
   void put(void* payload, uint64_t simulated_size_in_bytes, double timeout);
 
-  /** Creates (but don't start) a data reception onto that mailbox */
+  /** Creates (but don't start) a data reception onto that mailbox.
+   * @verbatim embed:rst:inline You probably want to use :cpp:func:`simgrid::s4u::Comm::set_dst_data` and friends before
+   * starting that activity. @endverbatim  */
   CommPtr get_init();
   /** Creates and start an async data reception to that mailbox */
   template <typename T> CommPtr get_async(T** data);
+  /** Creates and start an async data reception to that mailbox. Since the data location is not provided, you'll have to
+   * use Comm::get_payload once the comm terminates */
+  CommPtr get_async();
 
   /** Blocking data reception */
   template <typename T> T* get();
@@ -123,49 +133,38 @@ public:
   template <typename T> T* get(double timeout);
   template <typename T> std::unique_ptr<T> get_unique(double timeout) { return std::unique_ptr<T>(get<T>(timeout)); }
 
-#ifndef DOXYGEN
-  XBT_ATTRIB_DEPRECATED_v331("Please use typed template Mailbox::get_async<>()") CommPtr get_async(void** data);
-  XBT_ATTRIB_DEPRECATED_v331("Please use typed template Mailbox::get<>()") void* get();
-  XBT_ATTRIB_DEPRECATED_v331("Please use typed template Mailbox::get<>()") void* get(double timeout);
-#endif
+  void clear();
 };
 
 template <typename T> CommPtr Mailbox::get_async(T** data)
 {
   CommPtr res = get_init()->set_dst_data(reinterpret_cast<void**>(data), sizeof(void*));
-  res->vetoable_start();
+  res->start();
   return res;
 }
 
 template <typename T> T* Mailbox::get()
 {
   T* res = nullptr;
-  get_async<T>(&res)->wait();
+
+  CommPtr comm = get_async<T>(&res);
+  try {
+    comm->wait();
+  } catch (simgrid::TimeoutException&) {
+    comm->cancel();
+    /* Rethrowing the original exception segfaults in parallel tests */
+    throw TimeoutException(XBT_THROW_POINT, "Timeouted");
+  }
+
   return res;
 }
 
 template <typename T> T* Mailbox::get(double timeout)
 {
   T* res = nullptr;
-  get_async<T>(&res)->wait_for(timeout);
+  get_async<T>(&res)->wait_for_or_cancel(timeout);
   return res;
 }
-
-#ifndef DOXYGEN
-inline CommPtr Mailbox::get_async(void** data) // XBT_ATTRIB_DEPRECATED_v331
-{
-  return get_async<void>(data);
-}
-inline void* Mailbox::get() // XBT_ATTRIB_DEPRECATED_v331
-{
-  return get<void>();
-}
-inline void* Mailbox::get(double timeout) // XBT_ATTRIB_DEPRECATED_v331
-{
-  return get<void>(timeout);
-}
-#endif
-} // namespace s4u
-} // namespace simgrid
+} // namespace simgrid::s4u
 
 #endif /* SIMGRID_S4U_MAILBOX_HPP */
