@@ -1,13 +1,13 @@
-/* xbt.h - Public interface to the xbt (gras's toolbox)                   */
+/* xbt.h - Public interface to the xbt (SimGrid's toolbox)                  */
 
-/* Copyright (c) 2004, 2005, 2006, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2024. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#ifndef xbt_H
-#define xbt_H
+#ifndef XBT_H
+#define XBT_H
 
 #include <xbt/misc.h>
 #include <xbt/sysdep.h>
@@ -17,19 +17,17 @@
 #include <xbt/asserts.h>
 #include <xbt/log.h>
 
-#include <xbt/module.h>
-#include <xbt/strbuff.h>
-
 #include <xbt/dynar.h>
 #include <xbt/dict.h>
-#include <xbt/set.h>
-#include <xbt/swag.h>
-#include <xbt/heap.h>
-#include <xbt/queue.h>
-#include <xbt/parmap.h>
 
-#include <xbt/peer.h>
 #include <xbt/config.h>
-#include <xbt/cunit.h>
+#ifdef __cplusplus
+extern "C++"
+{
+#include <xbt/config.hpp>
+}
+#endif
 
-#endif                          /* xbt_H */
+#include <xbt/xbt_os_time.h>
+
+#endif /* XBT_H */
