@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -14,9 +14,9 @@
 #include <xbt/graph.h>
 
 #include "src/instr/instr_private.hpp"
-#include "src/surf/cpu_interface.hpp"
-#include "src/surf/network_interface.hpp"
-#include "src/surf/surf_interface.hpp"
+#include "src/kernel/activity/ExecImpl.hpp"
+#include "src/kernel/resource/CpuImpl.hpp"
+#include "src/kernel/resource/NetworkModel.hpp"
 
 #include <fstream>
 
@@ -24,7 +24,7 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(instr_routing, instr, "Tracing platform hierarch
 
 std::string instr_pid(simgrid::s4u::Actor const& proc)
 {
-  return std::string(proc.get_name()) + "-" + std::to_string(proc.get_pid());
+  return proc.get_name() + "-" + std::to_string(proc.get_pid());
 }
 
 static simgrid::instr::Container* lowestCommonAncestor(const simgrid::instr::Container* a1,
@@ -50,12 +50,9 @@ static simgrid::instr::Container* lowestCommonAncestor(const simgrid::instr::Con
   int j = static_cast<int>(ancestors_a2.size()) - 1;
   while (i >= 0 && j >= 0) {
     simgrid::instr::Container* a1p       = ancestors_a1.at(i);
-    const simgrid::instr::Container* a2p = ancestors_a2.at(j);
-    if (a1p == a2p) {
-      p = a1p;
-    } else {
+    if (a1p != ancestors_a2.at(j))
       break;
-    }
+    p = a1p;
     i--;
     j--;
   }
@@ -131,8 +128,7 @@ static void recursiveGraphExtraction(const simgrid::s4u::NetZone* netzone, const
   std::map<std::string, xbt_edge_t, std::less<>> edges;
 
   netzone->get_impl()->get_graph(graph, &nodes, &edges);
-  for (auto const& elm : edges) {
-    const xbt_edge* edge = elm.second;
+  for (auto const& [_, edge] : edges) {
     linkContainers(simgrid::instr::Container::by_name(static_cast<const char*>(edge->src->data)),
                    simgrid::instr::Container::by_name(static_cast<const char*>(edge->dst->data)), filter);
   }
@@ -146,13 +142,13 @@ static void recursiveNewVariableType(const std::string& new_typename, const std:
                                      simgrid::instr::Type* root)
 {
   if (root->get_name() == "HOST" || root->get_name() == "VM")
-    root->by_name_or_create(std::string("p") + new_typename, color);
+    root->by_name_or_create("p" + new_typename, color);
 
   if (root->get_name() == "LINK")
-    root->by_name_or_create(std::string("b") + new_typename, color);
+    root->by_name_or_create("b" + new_typename, color);
 
-  for (auto const& elm : root->get_children()) {
-    recursiveNewVariableType(new_typename, color, elm.second.get());
+  for (auto const& [_, child] : root->get_children()) {
+    recursiveNewVariableType(new_typename, color, child.get());
   }
 }
 
@@ -167,8 +163,8 @@ static void recursiveNewUserVariableType(const std::string& parent_type, const s
   if (root->get_name() == parent_type) {
     root->by_name_or_create(new_typename, color);
   }
-  for (auto const& elm : root->get_children())
-    recursiveNewUserVariableType(parent_type, new_typename, color, elm.second.get());
+  for (auto const& [_, child] : root->get_children())
+    recursiveNewUserVariableType(parent_type, new_typename, color, child.get());
 }
 
 void instr_new_user_variable_type(const std::string& parent_type, const std::string& new_typename,
@@ -183,8 +179,8 @@ static void recursiveNewUserStateType(const std::string& parent_type, const std:
   if (root->get_name() == parent_type)
     root->by_name_or_create<simgrid::instr::StateType>(new_typename);
 
-  for (auto const& elm : root->get_children())
-    recursiveNewUserStateType(parent_type, new_typename, elm.second.get());
+  for (auto const& [_, child] : root->get_children())
+    recursiveNewUserStateType(parent_type, new_typename, child.get());
 }
 
 void instr_new_user_state_type(const std::string& parent_type, const std::string& new_typename)
@@ -198,8 +194,8 @@ static void recursiveNewValueForUserStateType(const std::string& type_name, cons
   if (root->get_name() == type_name)
     static_cast<simgrid::instr::StateType*>(root)->add_entity_value(val, color);
 
-  for (auto const& elm : root->get_children())
-    recursiveNewValueForUserStateType(type_name, val, color, elm.second.get());
+  for (auto const& [_, child] : root->get_children())
+    recursiveNewValueForUserStateType(type_name, val, color, child.get());
 }
 
 void instr_new_value_for_user_state_type(const std::string& type_name, const char* value, const std::string& color)
@@ -207,9 +203,13 @@ void instr_new_value_for_user_state_type(const std::string& type_name, const cha
   recursiveNewValueForUserStateType(type_name, value, color, simgrid::instr::Container::get_root()->get_type());
 }
 
-namespace simgrid {
-namespace instr {
+namespace simgrid::instr {
 
+/** @brief Creates a file with the topology of the platform file used for the simulator.
+ *
+ *  The graph topology will have the following properties: all hosts, links and routers of the platform file are mapped
+ *  to graph nodes; routes are mapped to edges. The platform's zones are not represented in the output.
+ */
 void platform_graph_export_graphviz(const std::string& output_filename)
 {
   auto* g     = xbt_graph_new_graph(0, nullptr);
@@ -222,32 +222,53 @@ void platform_graph_export_graphviz(const std::string& output_filename)
   xbt_assert(not fs.fail(), "Failed to open %s", output_filename.c_str());
 
   if (g->directed)
-    fs << "digraph test {" << std::endl;
+    fs << "digraph test {\n";
   else
-    fs << "graph test {" << std::endl;
+    fs << "graph test {\n";
 
-  fs << "  graph [overlap=scale]" << std::endl;
+  fs << "  graph [overlap=scale]\n";
 
-  fs << "  node [shape=box, style=filled]" << std::endl;
-  fs << "  node [width=.3, height=.3, style=filled, color=skyblue]" << std::endl << std::endl;
+  fs << "  node [shape=box, style=filled]\n";
+  fs << "  node [width=.3, height=.3, style=filled, color=skyblue]\n\n";
 
-  for (auto const& elm : nodes)
-    fs << "  \"" << elm.first << "\";" << std::endl;
+  for (auto const& [node, _] : nodes)
+    fs << "  \"" << node << "\";\n";
 
-  for (auto const& elm : edges) {
-    const char* src_s = static_cast<char*>(elm.second->src->data);
-    const char* dst_s = static_cast<char*>(elm.second->dst->data);
+  for (auto const& [_, edge] : edges) {
+    const char* src_s = static_cast<char*>(edge->src->data);
+    const char* dst_s = static_cast<char*>(edge->dst->data);
     if (g->directed)
-      fs << "  \"" << src_s << "\" -> \"" << dst_s << "\";" << std::endl;
+      fs << "  \"" << src_s << "\" -> \"" << dst_s << "\";\n";
     else
-      fs << "  \"" << src_s << "\" -- \"" << dst_s << "\";" << std::endl;
+      fs << "  \"" << src_s << "\" -- \"" << dst_s << "\";\n";
   }
-  fs << "}" << std::endl;
+  fs << "}\n";
   fs.close();
 
   xbt_graph_free_graph(g, xbt_free_f, xbt_free_f, nullptr);
 }
 
+void platform_graph_export_csv(const std::string& output_filename)
+{
+  auto* g         = xbt_graph_new_graph(0, nullptr);
+  std::map<std::string, xbt_node_t, std::less<>> nodes;
+  std::map<std::string, xbt_edge_t, std::less<>> edges;
+  s4u::Engine::get_instance()->get_netzone_root()->extract_xbt_graph(g, &nodes, &edges);
+
+  std::ofstream fs;
+  fs.open(output_filename, std::ofstream::out);
+  xbt_assert(not fs.fail(), "Failed to open %s", output_filename.c_str());
+
+  fs << "src,dst" << std::endl;
+  for (auto const& [_, edge] : edges) {
+    const char* src_s = static_cast<char*>(edge->src->data);
+    const char* dst_s = static_cast<char*>(edge->dst->data);
+    fs << src_s << "," << dst_s << "\n";
+  }
+  fs.close();
+  xbt_graph_free_graph(g, xbt_free_f, xbt_free_f, nullptr);
+}
+
 /* Callbacks */
 static std::vector<NetZoneContainer*> currentContainer; /* push and pop, used only in creation */
 static void on_netzone_creation(s4u::NetZone const& netzone)
@@ -327,6 +348,15 @@ static void on_host_creation(s4u::Host const& host)
     root->get_type()->by_name_or_create("MIGRATE_LINK", mpi, mpi);
     mpi->by_name_or_create<StateType>("MIGRATE_STATE");
   }
+
+   if (TRACE_actor_is_enabled()) {
+    auto* host_type = container->get_type();
+    auto* state     = host_type->by_name_or_create<StateType>("HOST_STATE");
+    state->set_calling_container(container);
+    state->add_entity_value("receive", "1 0 0");
+    state->add_entity_value("send", "0 0 1");
+    state->add_entity_value("execute", "0 1 1");
+  }
 }
 
 static void on_action_state_change(kernel::resource::Action const& action,
@@ -338,20 +368,21 @@ static void on_action_state_change(kernel::resource::Action const& action,
     double value = action.get_rate() * action.get_variable()->get_constraint_weight(i);
     /* Beware of composite actions: ptasks put links and cpus together. Extra pb: we cannot dynamic_cast from void* */
     kernel::resource::Resource* resource = action.get_variable()->get_constraint(i)->get_id();
-    const kernel::resource::CpuImpl* cpu = dynamic_cast<kernel::resource::CpuImpl*>(resource);
-
-    if (cpu != nullptr)
+    if (const auto* cpu = dynamic_cast<kernel::resource::CpuImpl*>(resource))
       resource_set_utilization("HOST", "speed_used", cpu->get_cname(), action.get_category(), value,
                                action.get_last_update(), simgrid_get_clock() - action.get_last_update());
 
-    const kernel::resource::LinkImpl* link = dynamic_cast<kernel::resource::LinkImpl*>(resource);
-
-    if (link != nullptr)
+    else if (const auto* link = dynamic_cast<kernel::resource::StandardLinkImpl*>(resource))
       resource_set_utilization("LINK", "bandwidth_used", link->get_cname(), action.get_category(), value,
                                action.get_last_update(), simgrid_get_clock() - action.get_last_update());
   }
 }
 
+static void on_activity_suspend_resume(s4u::Activity const& activity)
+{
+  on_action_state_change(*activity.get_impl()->model_action_, /*ignored*/ kernel::resource::Action::State::STARTED);
+}
+
 static void on_platform_created()
 {
   currentContainer.clear();
@@ -422,93 +453,118 @@ void define_callbacks()
   // always need the callbacks to zones (we need only the root zone), to create the rootContainer and the rootType
   // properly
   if (TRACE_needs_platform()) {
-    s4u::Engine::on_platform_created.connect(on_platform_created);
-    s4u::Host::on_creation.connect(on_host_creation);
-    s4u::Host::on_speed_change.connect([](s4u::Host const& host) {
+    s4u::Engine::on_platform_created_cb(on_platform_created);
+    s4u::Host::on_creation_cb(on_host_creation);
+    s4u::Host::on_speed_change_cb([](s4u::Host const& host) {
       Container::by_name(host.get_name())
           ->get_variable("speed")
           ->set_event(simgrid_get_clock(), host.get_core_count() * host.get_available_speed());
     });
-    s4u::Link::on_creation.connect(on_link_creation);
-    s4u::Link::on_bandwidth_change.connect([](s4u::Link const& link) {
+    s4u::Link::on_creation_cb(on_link_creation);
+    s4u::Link::on_bandwidth_change_cb([](s4u::Link const& link) {
       Container::by_name(link.get_name())
           ->get_variable("bandwidth")
-          ->set_event(simgrid_get_clock(), sg_bandwidth_factor * link.get_bandwidth());
+          ->set_event(
+              simgrid_get_clock(),
+              static_cast<kernel::resource::NetworkModel*>(link.get_impl()->get_model())->get_bandwidth_factor() *
+                  link.get_bandwidth());
     });
-    s4u::NetZone::on_seal.connect([](s4u::NetZone const& /*netzone*/) { currentContainer.pop_back(); });
     kernel::routing::NetPoint::on_creation.connect([](kernel::routing::NetPoint const& netpoint) {
       if (netpoint.is_router())
         new RouterContainer(netpoint.get_name(), currentContainer.back());
     });
   }
 
-  s4u::NetZone::on_creation.connect(on_netzone_creation);
+  s4u::NetZone::on_creation_cb(on_netzone_creation);
 
-  kernel::resource::CpuAction::on_state_change.connect(on_action_state_change);
-  s4u::Link::on_communication_state_change.connect(on_action_state_change);
+  s4u::Host::on_exec_state_change_cb(on_action_state_change);
+  s4u::Link::on_communication_state_change_cb(on_action_state_change);
+  s4u::Exec::on_suspend_cb(on_activity_suspend_resume);
+  s4u::Exec::on_resume_cb(on_activity_suspend_resume);
 
   if (TRACE_actor_is_enabled()) {
-    s4u::Actor::on_creation.connect(on_actor_creation);
-    s4u::Actor::on_destruction.connect([](s4u::Actor const& actor) {
-      auto container = Container::by_name_or_null(instr_pid(actor));
-      if (container != nullptr)
+    s4u::Actor::on_creation_cb(on_actor_creation);
+    s4u::Actor::on_destruction_cb([](s4u::Actor const& actor) {
+      if (auto* container = Container::by_name_or_null(instr_pid(actor)))
         container->remove_from_parent();
     });
-    s4u::Actor::on_suspend.connect([](s4u::Actor const& actor) {
+    s4u::Actor::on_suspend_cb([](s4u::Actor const& actor) {
       Container::by_name(instr_pid(actor))->get_state("ACTOR_STATE")->push_event("suspend");
     });
-    s4u::Actor::on_resume.connect(
+    s4u::Actor::on_resume_cb(
         [](s4u::Actor const& actor) { Container::by_name(instr_pid(actor))->get_state("ACTOR_STATE")->pop_event(); });
-    s4u::Actor::on_sleep.connect([](s4u::Actor const& actor) {
+    s4u::Actor::on_sleep_cb([](s4u::Actor const& actor) {
       Container::by_name(instr_pid(actor))->get_state("ACTOR_STATE")->push_event("sleep");
     });
-    s4u::Actor::on_wake_up.connect(
+    s4u::Actor::on_wake_up_cb(
         [](s4u::Actor const& actor) { Container::by_name(instr_pid(actor))->get_state("ACTOR_STATE")->pop_event(); });
-    s4u::Exec::on_start.connect([](s4u::Exec const&) {
-      Container::by_name(instr_pid(*s4u::Actor::self()))->get_state("ACTOR_STATE")->push_event("execute");
+
+    s4u::Exec::on_start_cb([](s4u::Exec const& e) {
+      std::string pid = instr_pid(*s4u::Actor::self());
+      if (pid == "-0") //Exec is launched directly by Maestro, use the host as container
+        Container::by_name(e.get_host()->get_name())->get_state("HOST_STATE")->push_event("execute");
+      else
+        Container::by_name(pid)->get_state("ACTOR_STATE")->push_event("execute");
     });
-    s4u::Exec::on_completion.connect([](s4u::Exec const&) {
-      Container::by_name(instr_pid(*s4u::Actor::self()))->get_state("ACTOR_STATE")->pop_event();
+
+    s4u::Exec::on_completion_cb([](const s4u::Exec& e) {
+      std::string pid = instr_pid(*s4u::Actor::self());
+      if (pid == "-0") //Exec is launched directly by Maestro, use the host as container
+        Container::by_name(e.get_host()->get_name())->get_state("HOST_STATE")->pop_event();
+      else
+        Container::by_name(pid)->get_state("ACTOR_STATE")->pop_event();
     });
-    s4u::Comm::on_start.connect([](s4u::Comm const&, bool is_sender) {
-      Container::by_name(instr_pid(*s4u::Actor::self()))
-          ->get_state("ACTOR_STATE")
-          ->push_event(is_sender ? "send" : "receive");
+
+    s4u::Comm::on_completion_cb([](const s4u::Comm& c) {
+      if (c.get_sender()) {
+        Container::by_name(instr_pid(*c.get_sender()))->get_state("ACTOR_STATE")->pop_event();
+        Container::by_name(instr_pid(*c.get_receiver()))->get_state("ACTOR_STATE")->pop_event();
+      } else if(c.get_source()) {
+        Container::by_name(c.get_source()->get_name())->get_state("HOST_STATE")->pop_event();
+        Container::by_name(c.get_destination()->get_name())->get_state("HOST_STATE")->pop_event();
+      }
     });
-    s4u::Comm::on_completion.connect([](s4u::Comm const&) {
-      Container::by_name(instr_pid(*s4u::Actor::self()))->get_state("ACTOR_STATE")->pop_event();
+    s4u::Comm::on_start_cb([](s4u::Comm const& c) {
+      std::string pid = instr_pid(*s4u::Actor::self());
+      if (pid == "-0") { //Comm is launched directly by Maestro, use the host as container
+        Container::by_name(c.get_source()->get_name())->get_state("HOST_STATE")->push_event("start");
+        Container::by_name(c.get_destination()->get_name())->get_state("HOST_STATE")->push_event("start");
+      }
     });
-    s4u::Actor::on_host_change.connect(on_actor_host_change);
+    s4u::Comm::on_send_cb([](s4u::Comm const&) {
+      Container::by_name(instr_pid(*s4u::Actor::self()))->get_state("ACTOR_STATE")->push_event("send");
+    });
+    s4u::Comm::on_recv_cb([](s4u::Comm const&) {
+      Container::by_name(instr_pid(*s4u::Actor::self()))->get_state("ACTOR_STATE")->push_event("receive");
+    });
+    s4u::Actor::on_host_change_cb(on_actor_host_change);
   }
 
   if (TRACE_smpi_is_enabled() && TRACE_smpi_is_computing()) {
-    s4u::Exec::on_start.connect([](s4u::Exec const& exec) {
-      Container::by_name(std::string("rank-") + std::to_string(s4u::Actor::self()->get_pid()))
+    s4u::Exec::on_start_cb([](s4u::Exec const& exec) {
+      Container::by_name("rank-" + std::to_string(s4u::Actor::self()->get_pid()))
           ->get_state("MPI_STATE")
           ->push_event("computing", new CpuTIData("compute", exec.get_cost()));
     });
-    s4u::Exec::on_completion.connect([](s4u::Exec const&) {
-      Container::by_name(std::string("rank-") + std::to_string(s4u::Actor::self()->get_pid()))
-          ->get_state("MPI_STATE")
-          ->pop_event();
+    s4u::Exec::on_completion_cb([](const s4u::Exec&) {
+      Container::by_name("rank-" + std::to_string(s4u::Actor::self()->get_pid()))->get_state("MPI_STATE")->pop_event();
     });
   }
 
   if (TRACE_vm_is_enabled()) {
-    s4u::Host::on_creation.connect(on_vm_creation);
-    s4u::VirtualMachine::on_start.connect([](s4u::VirtualMachine const& vm) {
+    s4u::Host::on_creation_cb(on_vm_creation);
+    s4u::VirtualMachine::on_start_cb([](s4u::VirtualMachine const& vm) {
       Container::by_name(vm.get_name())->get_state("VM_STATE")->push_event("start");
     });
-    s4u::VirtualMachine::on_started.connect(
+    s4u::VirtualMachine::on_started_cb(
         [](s4u::VirtualMachine const& vm) { Container::by_name(vm.get_name())->get_state("VM_STATE")->pop_event(); });
-    s4u::VirtualMachine::on_suspend.connect([](s4u::VirtualMachine const& vm) {
+    s4u::VirtualMachine::on_suspend_cb([](s4u::VirtualMachine const& vm) {
       Container::by_name(vm.get_name())->get_state("VM_STATE")->push_event("suspend");
     });
-    s4u::VirtualMachine::on_resume.connect(
+    s4u::VirtualMachine::on_resume_cb(
         [](s4u::VirtualMachine const& vm) { Container::by_name(vm.get_name())->get_state("VM_STATE")->pop_event(); });
-    s4u::Host::on_destruction.connect(
+    s4u::Host::on_destruction_cb(
         [](s4u::Host const& host) { Container::by_name(host.get_name())->remove_from_parent(); });
   }
 }
-} // namespace instr
-} // namespace simgrid
+} // namespace simgrid::instr
