@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -24,6 +24,7 @@ XBT_PRIVATE void TRACE_smpi_init(aid_t pid, const std::string& calling_func);
 
 class smpi_trace_call_location_t {
 public:
+  std::string func_call;
   std::string filename;
   int linenumber = 0;
 
@@ -35,6 +36,8 @@ public:
     return previous_filename + ':' + std::to_string(previous_linenumber) + ':' + filename + ':' +
            std::to_string(linenumber);
   }
+
+  std::string get_call_location() const { return filename + ":" + std::to_string(linenumber) + ":" + func_call + "()"; }
 };
 
 #endif
