@@ -1,4 +1,4 @@
-/* Copyright (c) 2009-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2009-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -11,13 +11,11 @@
 #include <functional>
 #include <vector>
 
-#include <xbt/parmap.hpp>
+#include "src/xbt/parmap.hpp"
 
 #include "src/kernel/context/ContextSwapped.hpp"
 
-namespace simgrid {
-namespace kernel {
-namespace context {
+namespace simgrid::kernel::context {
 
 /** @brief Fast context switching inspired from SystemV ucontexts.
   *
@@ -37,10 +35,9 @@ private:
 
 class RawContextFactory : public SwappedContextFactory {
 public:
+  const char* get_name() const override { return "raw"; }
   RawContext* create_context(std::function<void()>&& code, actor::ActorImpl* actor) override;
 };
-} // namespace context
-} // namespace kernel
-} // namespace simgrid
+} // namespace simgrid::kernel::context
 
 #endif
