@@ -1,18 +1,18 @@
-/* Copyright (c) 2009-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2009-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 /* \file UContext.cpp Context switching with ucontexts from System V        */
 
-#include "mc/mc.h"
 #include "simgrid/Exception.hpp"
 #include "src/kernel/actor/ActorImpl.hpp"
-#include "src/mc/mc_ignore.hpp"
+#include "src/mc/mc.h"
+#include "src/mc/remote/AppSide.hpp"
 
 #include "ContextUnix.hpp"
 
-XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(simix_context);
+XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(ker_context);
 
 /** Up to two integers may be needed to store a pointer on the system we target */
 constexpr int CTX_ADDR_LEN = 2;
@@ -32,9 +32,7 @@ XBT_ATTRIB_NORETURN static void sysv_ctx_wrapper(int i1, int i2)
 }
 }
 
-namespace simgrid {
-namespace kernel {
-namespace context {
+namespace simgrid::kernel::context {
 
 // UContextFactory
 UContext* UContextFactory::create_context(std::function<void()>&& code, actor::ActorImpl* actor)
@@ -62,12 +60,6 @@ UContext::UContext(std::function<void()>&& code, actor::ActorImpl* actor, Swappe
     UContext* arg = this;
     memcpy(ctx_addr, &arg, sizeof arg);
     makecontext(&this->uc_, (void (*)())sysv_ctx_wrapper, 2, ctx_addr[0], ctx_addr[1]);
-
-#if SIMGRID_HAVE_MC
-    if (MC_is_active()) {
-      MC_register_stack_area(get_stack(), &(this->uc_), stack_size);
-    }
-#endif
   }
 }
 
@@ -83,6 +75,4 @@ XBT_PRIVATE ContextFactory* sysv_factory()
   XBT_VERB("Activating SYSV context factory");
   return new UContextFactory();
 }
-} // namespace context
-} // namespace kernel
-} // namespace simgrid
+} // namespace simgrid::kernel::context
