@@ -1,22 +1,21 @@
-/* Copyright (c) 2014-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2014-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "simgrid/kernel/routing/DragonflyZone.hpp"
 #include "simgrid/kernel/routing/NetPoint.hpp"
-#include "src/surf/network_interface.hpp"
+#include "src/kernel/resource/NetworkModel.hpp"
 
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <numeric>
 #include <string>
 
-XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_route_cluster_dragonfly, surf_route_cluster, "Dragonfly Routing part of surf");
+XBT_LOG_NEW_DEFAULT_SUBCATEGORY(ker_routing_dragonfly, ker_platform, "Kernel Dragonfly Routing");
 
 namespace simgrid {
-namespace kernel {
-namespace routing {
+namespace kernel::routing {
 
 DragonflyZone::DragonflyZone(const std::string& name) : ClusterBase(name) {}
 
@@ -33,15 +32,6 @@ DragonflyZone::Coords DragonflyZone::rankId_to_coords(unsigned long rankId) cons
   return coords;
 }
 
-void DragonflyZone::rankId_to_coords(int rankId, unsigned long coords[4]) const // XBT_ATTRIB_DEPRECATED_v330
-{
-  const auto s_coords = rankId_to_coords(rankId);
-  coords[0]           = s_coords.group;
-  coords[1]           = s_coords.chassis;
-  coords[2]           = s_coords.blade;
-  coords[3]           = s_coords.node;
-}
-
 void DragonflyZone::set_link_characteristics(double bw, double lat, s4u::Link::SharingPolicy sharing_policy)
 {
   ClusterBase::set_link_characteristics(bw, lat, sharing_policy);
@@ -83,14 +73,14 @@ s4u::DragonflyParams DragonflyZone::parse_topo_parameters(const std::string& top
   try {
     n_groups = std::stoi(tmp[0]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Invalid number of groups:") + tmp[0]);
+    throw std::invalid_argument("Invalid number of groups:" + tmp[0]);
   }
 
   unsigned int n_blue;
   try {
     n_blue = std::stoi(tmp[1]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Invalid number of links for the blue level:") + tmp[1]);
+    throw std::invalid_argument("Invalid number of links for the blue level:" + tmp[1]);
   }
 
   // Black network : number of chassis/group, number of links between each router on the black network
@@ -102,14 +92,14 @@ s4u::DragonflyParams DragonflyZone::parse_topo_parameters(const std::string& top
   try {
     n_chassis = std::stoi(tmp[0]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Invalid number of chassis:") + tmp[0]);
+    throw std::invalid_argument("Invalid number of chassis:" + tmp[0]);
   }
 
   unsigned int n_black;
   try {
     n_black = std::stoi(tmp[1]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Invalid number of links for the black level:") + tmp[1]);
+    throw std::invalid_argument("Invalid number of links for the black level:" + tmp[1]);
   }
 
   // Green network : number of blades/chassis, number of links between each router on the green network
@@ -121,14 +111,14 @@ s4u::DragonflyParams DragonflyZone::parse_topo_parameters(const std::string& top
   try {
     n_routers = std::stoi(tmp[0]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Invalid number of routers:") + tmp[0]);
+    throw std::invalid_argument("Invalid number of routers:" + tmp[0]);
   }
 
   unsigned int n_green;
   try {
     n_green = std::stoi(tmp[1]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Invalid number of links for the green level:") + tmp[1]);
+    throw std::invalid_argument("Invalid number of links for the green level:" + tmp[1]);
   }
 
   // The last part of topo_parameters should be the number of nodes per blade
@@ -136,7 +126,7 @@ s4u::DragonflyParams DragonflyZone::parse_topo_parameters(const std::string& top
   try {
     n_nodes = std::stoi(parameters[3]);
   } catch (const std::invalid_argument&) {
-    throw std::invalid_argument(std::string("Last parameter is not the amount of nodes per blade:") + parameters[3]);
+    throw std::invalid_argument("Last parameter is not the amount of nodes per blade:" + parameters[3]);
   }
   return s4u::DragonflyParams({n_groups, n_blue}, {n_chassis, n_black}, {n_routers, n_green}, n_nodes);
 }
@@ -153,8 +143,8 @@ void DragonflyZone::generate_routers(const s4u::ClusterCallbacks& set_callbacks)
   unsigned long id = 2UL * num_groups_ * num_chassis_per_group_ * num_blades_per_chassis_ * num_nodes_per_blade_;
   /* get limiter for this router */
   auto get_limiter = [this, &id, &set_callbacks](unsigned int i, unsigned int j,
-                                                 unsigned int k) -> resource::LinkImpl* {
-    kernel::resource::LinkImpl* limiter = nullptr;
+                                                 unsigned int k) -> resource::StandardLinkImpl* {
+    kernel::resource::StandardLinkImpl* limiter = nullptr;
     if (set_callbacks.limiter) {
       id--;
       const auto* s4u_link =
@@ -176,8 +166,8 @@ void DragonflyZone::generate_routers(const s4u::ClusterCallbacks& set_callbacks)
   }
 }
 
-void DragonflyZone::generate_link(const std::string& id, int numlinks, resource::LinkImpl** linkup,
-                                  resource::LinkImpl** linkdown)
+void DragonflyZone::generate_link(const std::string& id, int numlinks, resource::StandardLinkImpl** linkup,
+                                  resource::StandardLinkImpl** linkdown)
 {
   XBT_DEBUG("Generating link %s", id.c_str());
   *linkup   = nullptr;
@@ -198,8 +188,8 @@ void DragonflyZone::generate_link(const std::string& id, int numlinks, resource:
 void DragonflyZone::generate_links()
 {
   static int uniqueId = 0;
-  resource::LinkImpl* linkup;
-  resource::LinkImpl* linkdown;
+  resource::StandardLinkImpl* linkup;
+  resource::StandardLinkImpl* linkdown;
 
   unsigned int numRouters = num_groups_ * num_chassis_per_group_ * num_blades_per_chassis_;
 
@@ -288,7 +278,7 @@ void DragonflyZone::get_local_route(const NetPoint* src, const NetPoint* dst, Ro
            dst->id());
 
   if ((src->id() == dst->id()) && has_loopback()) {
-    resource::LinkImpl* uplink = get_uplink_from(node_pos(src->id()));
+    resource::StandardLinkImpl* uplink = get_uplink_from(node_pos(src->id()));
 
     add_link_latency(route->link_list_, uplink, latency);
     return;
@@ -375,8 +365,7 @@ void DragonflyZone::get_local_route(const NetPoint* src, const NetPoint* dst, Ro
   route->gw_src_ = get_gateway(src->id());
   route->gw_dst_ = get_gateway(dst->id());
 }
-} // namespace routing
-} // namespace kernel
+} // namespace kernel::routing
 
 namespace s4u {
 DragonflyParams::DragonflyParams(const std::pair<unsigned int, unsigned int>& groups,
