@@ -1,4 +1,4 @@
-/* Copyright (c) 2014-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2014-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -6,8 +6,8 @@
 #include <simgrid/kernel/routing/FatTreeZone.hpp>
 #include <simgrid/kernel/routing/NetPoint.hpp>
 
-#include "src/surf/network_interface.hpp"
-#include "src/surf/xml/platf.hpp" // surf_parse_error() and surf_parse_assert()
+#include "src/kernel/resource/NetworkModel.hpp"
+#include "src/kernel/xml/platf.hpp" // simgrid_parse_error() and simgrid_parse_assert()
 
 #include <fstream>
 #include <numeric>
@@ -17,11 +17,10 @@
 #include <boost/algorithm/string/classification.hpp>
 #include <boost/algorithm/string/split.hpp>
 
-XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_route_fat_tree, surf, "Routing for fat trees");
+XBT_LOG_NEW_DEFAULT_SUBCATEGORY(ker_routing_fat_tree, ker_platform, "Kernel Fat-Tree Routing");
 
 namespace simgrid {
-namespace kernel {
-namespace routing {
+namespace kernel::routing {
 
 bool FatTreeZone::is_in_sub_tree(const FatTreeNode* root, const FatTreeNode* node) const
 {
@@ -78,8 +77,8 @@ void FatTreeZone::get_local_route(const NetPoint* src, const NetPoint* dst, Rout
     for (unsigned int i = 0; i < currentNode->level; i++)
       d /= this->num_parents_per_node_[i];
 
-    int k = this->num_parents_per_node_[currentNode->level];
-    d     = d % k;
+    int k = this->num_parents_per_node_[currentNode->level] * this->num_port_lower_level_[currentNode->level];
+    d = d % k;
 
     if (currentNode->limiter_link_)
       into->link_list_.push_back(currentNode->limiter_link_);
@@ -94,7 +93,9 @@ void FatTreeZone::get_local_route(const NetPoint* src, const NetPoint* dst, Rout
 
   // Down part
   while (currentNode != destination) {
-    for (unsigned int i = 0; i < currentNode->children.size(); i++) {
+    //pick cable when multiple parallels
+    int d = source->position % this->num_port_lower_level_[currentNode->level - 1];
+    for (unsigned int i = d * this->num_children_per_node_[currentNode->level - 1]; i < currentNode->children.size(); i++) {
       if (i % this->num_children_per_node_[currentNode->level - 1] == destination->label[currentNode->level - 1]) {
         add_link_latency(into->link_list_, currentNode->children[i]->down_link_, latency);
 
@@ -135,7 +136,7 @@ void FatTreeZone::do_seal()
   if (this->levels_ == 0) {
     return;
   }
-  if (not XBT_LOG_ISENABLED(surf_route_fat_tree, xbt_log_priority_debug)) {
+  if (not XBT_LOG_ISENABLED(ker_routing_fat_tree, xbt_log_priority_debug)) {
     return;
   }
 
@@ -192,7 +193,7 @@ bool FatTreeZone::are_related(FatTreeNode* parent, FatTreeNode* child) const
 {
   std::stringstream msgBuffer;
 
-  if (XBT_LOG_ISENABLED(surf_route_fat_tree, xbt_log_priority_debug)) {
+  if (XBT_LOG_ISENABLED(ker_routing_fat_tree, xbt_log_priority_debug)) {
     msgBuffer << "Are " << child->id << "(" << child->level << "," << child->position << ") <";
 
     for (unsigned int i = 0; i < this->levels_; i++) {
@@ -231,9 +232,10 @@ void FatTreeZone::generate_switches(const s4u::ClusterCallbacks& set_callbacks)
     this->nodes_by_level_[0] *= this->num_children_per_node_[i];
 
   if (this->nodes_by_level_[0] != this->nodes_.size()) {
-    surf_parse_error(std::string("The number of provided nodes does not fit with the wanted topology.") +
-                     " Please check your platform description (We need " + std::to_string(this->nodes_by_level_[0]) +
-                     "nodes, we got " + std::to_string(this->nodes_.size()));
+    simgrid_parse_error("The number of provided nodes does not fit with the wanted topology."
+                        " Please check your platform description (We need " +
+                        std::to_string(this->nodes_by_level_[0]) + "nodes, we got " +
+                        std::to_string(this->nodes_.size()));
   }
 
   for (unsigned int i = 0; i < this->levels_; i++) {
@@ -249,8 +251,8 @@ void FatTreeZone::generate_switches(const s4u::ClusterCallbacks& set_callbacks)
   }
 
   /* get limiter for this router */
-  auto get_limiter = [this, &set_callbacks](unsigned long i, unsigned long j, long id) -> resource::LinkImpl* {
-    kernel::resource::LinkImpl* limiter = nullptr;
+  auto get_limiter = [this, &set_callbacks](unsigned long i, unsigned long j, long id) -> resource::StandardLinkImpl* {
+    kernel::resource::StandardLinkImpl* limiter = nullptr;
     if (set_callbacks.limiter) {
       const auto* s4u_link = set_callbacks.limiter(get_iface(), {i + 1, j}, id);
       if (s4u_link) {
@@ -291,7 +293,7 @@ void FatTreeZone::generate_labels()
     }
 
     for (unsigned int j = 0; j < this->nodes_by_level_[i]; j++) {
-      if (XBT_LOG_ISENABLED(surf_route_fat_tree, xbt_log_priority_debug)) {
+      if (XBT_LOG_ISENABLED(ker_routing_fat_tree, xbt_log_priority_debug)) {
         std::stringstream msgBuffer;
 
         msgBuffer << "Assigning label <";
@@ -333,7 +335,7 @@ int FatTreeZone::get_level_position(const unsigned int level)
   return tempPosition;
 }
 
-void FatTreeZone::add_processing_node(int id, resource::LinkImpl* limiter, resource::LinkImpl* loopback)
+void FatTreeZone::add_processing_node(int id, resource::StandardLinkImpl* limiter, resource::StandardLinkImpl* loopback)
 {
   using std::make_pair;
   static int position = 0;
@@ -341,7 +343,7 @@ void FatTreeZone::add_processing_node(int id, resource::LinkImpl* limiter, resou
   position++;
   newNode->parents.resize(static_cast<size_t>(this->num_parents_per_node_[0]) * this->num_port_lower_level_[0]);
   newNode->label.resize(this->levels_);
-  this->compute_nodes_.insert(make_pair(id, newNode));
+  this->compute_nodes_.try_emplace(id, newNode);
   this->nodes_.emplace_back(newNode);
 }
 
@@ -414,7 +416,7 @@ s4u::FatTreeParams FatTreeZone::parse_topo_parameters(const std::string& topo_pa
   std::vector<unsigned int> count;
   boost::split(parameters, topo_parameters, boost::is_any_of(";"));
 
-  surf_parse_assert(
+  simgrid_parse_assert(
       parameters.size() == 4,
       "Fat trees are defined by the levels number and 3 vectors, see the documentation for more information.");
 
@@ -422,46 +424,46 @@ s4u::FatTreeParams FatTreeZone::parse_topo_parameters(const std::string& topo_pa
   try {
     n_lev = std::stoi(parameters[0]);
   } catch (const std::invalid_argument&) {
-    surf_parse_error(std::string("First parameter is not the amount of levels: ") + parameters[0]);
+    simgrid_parse_error("First parameter is not the amount of levels: " + parameters[0]);
   }
 
   // Then, a l-sized vector standing for the children number by level
   boost::split(tmp, parameters[1], boost::is_any_of(","));
-  surf_parse_assert(tmp.size() == n_lev, std::string("You specified ") + std::to_string(n_lev) +
-                                             " levels but the child count vector (the first one) contains " +
-                                             std::to_string(tmp.size()) + " levels.");
+  simgrid_parse_assert(tmp.size() == n_lev, "You specified " + std::to_string(n_lev) +
+                                                " levels but the child count vector (the first one) contains " +
+                                                std::to_string(tmp.size()) + " levels.");
 
   for (std::string const& level : tmp) {
     try {
       down.push_back(std::stoi(level));
     } catch (const std::invalid_argument&) {
-      surf_parse_error(std::string("Invalid child count: ") + level);
+      simgrid_parse_error("Invalid child count: " + level);
     }
   }
 
   // Then, a l-sized vector standing for the parents number by level
   boost::split(tmp, parameters[2], boost::is_any_of(","));
-  surf_parse_assert(tmp.size() == n_lev, std::string("You specified ") + std::to_string(n_lev) +
-                                             " levels but the parent count vector (the second one) contains " +
-                                             std::to_string(tmp.size()) + " levels.");
+  simgrid_parse_assert(tmp.size() == n_lev, "You specified " + std::to_string(n_lev) +
+                                                " levels but the parent count vector (the second one) contains " +
+                                                std::to_string(tmp.size()) + " levels.");
   for (std::string const& parent : tmp) {
     try {
       up.push_back(std::stoi(parent));
     } catch (const std::invalid_argument&) {
-      surf_parse_error(std::string("Invalid parent count: ") + parent);
+      simgrid_parse_error("Invalid parent count: " + parent);
     }
   }
 
   // Finally, a l-sized vector standing for the ports number with the lower level
   boost::split(tmp, parameters[3], boost::is_any_of(","));
-  surf_parse_assert(tmp.size() == n_lev, std::string("You specified ") + std::to_string(n_lev) +
-                                             " levels but the port count vector (the third one) contains " +
-                                             std::to_string(tmp.size()) + " levels.");
+  simgrid_parse_assert(tmp.size() == n_lev, "You specified " + std::to_string(n_lev) +
+                                                " levels but the port count vector (the third one) contains " +
+                                                std::to_string(tmp.size()) + " levels.");
   for (std::string const& port : tmp) {
     try {
       count.push_back(std::stoi(port));
     } catch (const std::invalid_argument&) {
-      throw std::invalid_argument(std::string("Invalid lower level port number:") + port);
+      throw std::invalid_argument("Invalid lower level port number:" + port);
     }
   }
   return s4u::FatTreeParams(n_lev, down, up, count);
@@ -488,8 +490,7 @@ void FatTreeZone::generate_dot_file(const std::string& filename) const
   file << "}";
   file.close();
 }
-} // namespace routing
-} // namespace kernel
+} // namespace kernel::routing
 
 namespace s4u {
 FatTreeParams::FatTreeParams(unsigned int n_levels, const std::vector<unsigned int>& down_links,
