@@ -1,9 +1,9 @@
-/* Copyright (c) 2017-2021. The SimGrid Team. All rights reserved.               */
+/* Copyright (c) 2017-2024. The SimGrid Team. All rights reserved.               */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "catch.hpp"
+#include "src/3rd-party/catch.hpp"
 
 #include "NetZone_test.hpp" // CreateHost callback
 #include "simgrid/kernel/routing/TorusZone.hpp"
