@@ -1,16 +1,12 @@
-/* Copyright (c) 2021. The SimGrid Team.
- * All rights reserved.                                                     */
+/* Copyright (c) 2021-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include <simgrid/kernel/Timer.hpp>
 #include <simgrid/s4u/Engine.hpp>
-#include <simgrid/simix.h>
 
-namespace simgrid {
-namespace kernel {
-namespace timer {
+namespace simgrid::kernel::timer {
 
 Timer* Timer::set(double date, xbt::Task<void()>&& callback)
 {
@@ -32,7 +28,6 @@ bool Timer::execute_all()
   bool result = false;
   while (not kernel_timers().empty() && s4u::Engine::get_clock() >= kernel_timers().top().first) {
     result = true;
-    // FIXME: make the timers being real callbacks (i.e. provide dispatchers that read and expand the args)
     Timer* timer = kernel_timers().top().second;
     kernel_timers().pop();
     timer->callback();
@@ -41,6 +36,4 @@ bool Timer::execute_all()
   return result;
 }
 
-} // namespace timer
-} // namespace kernel
-} // namespace simgrid
+} // namespace simgrid::kernel::timer
