@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -11,14 +11,15 @@
 #ifndef SIMGRID_MC_FORWARD_HPP
 #define SIMGRID_MC_FORWARD_HPP
 
-namespace simgrid {
-namespace mc {
+#include "simgrid/forward.h"
+#include <deque>
+
+namespace simgrid::mc {
 
 class PageStore;
 class ChunkedData;
-class ModelChecker;
 class AddressSpace;
-class RemoteProcess;
+class RemoteProcessMemory;
 class Snapshot;
 class ObjectInformation;
 class Member;
@@ -26,15 +27,17 @@ class Type;
 class Variable;
 class Transition;
 class Frame;
-class ActorInformation;
 
 class Session;
-class Checker;
+class Exploration;
+class SleepSetState;
 
-}
-}
+using StatePtr = boost::intrusive_ptr<State>;
+XBT_PUBLIC void intrusive_ptr_release(State* o);
+XBT_PUBLIC void intrusive_ptr_add_ref(State* o);
 
-// TODO, try to get rid of the global ModelChecker variable
-extern simgrid::mc::ModelChecker* mc_model_checker;
+using stack_t = std::deque<StatePtr>;
+
+} // namespace simgrid::mc
 
 #endif
