@@ -1,24 +1,28 @@
-/* Copyright (c) 2010-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid/plugins/dvfs.h"
-#include "simgrid/plugins/load.h"
-#include "simgrid/s4u/Engine.hpp"
-#include "src/internal_config.h" // HAVE_SMPI
-#include "src/kernel/activity/ExecImpl.hpp"
-#include "src/plugins/vm/VirtualMachineImpl.hpp"
-#if HAVE_SMPI
-#include "src/smpi/plugins/ampi/ampi.hpp"
-#endif
+#include <simgrid/plugins/dvfs.h>
+#include <simgrid/plugins/load.h>
+#include <simgrid/s4u/Actor.hpp>
+#include <simgrid/s4u/Host.hpp>
+#include <simgrid/s4u/VirtualMachine.hpp>
+#include <xbt/asserts.h>
 #include <xbt/config.hpp>
 
-#include <boost/algorithm/string.hpp>
+#include "src/internal_config.h" // HAVE_SMPI
+#include "src/kernel/activity/CommImpl.hpp"
+#include "src/kernel/resource/NetworkModel.hpp"
+#include "src/simgrid/module.hpp"
 #if HAVE_SMPI
 #include "src/smpi/include/smpi_request.hpp"
+#include "src/smpi/plugins/ampi/ampi.hpp"
 #endif
 
+#include <boost/algorithm/string.hpp>
+#include <string_view>
+
 SIMGRID_REGISTER_PLUGIN(host_dvfs, "Dvfs support", &sg_host_dvfs_plugin_init)
 
 static simgrid::config::Flag<double>
@@ -41,7 +45,7 @@ static simgrid::config::Flag<std::string> cfg_governor("plugin/dvfs/governor",
                                                              {"performance", "TODO: Doc"}, {"powersave", "TODO: Doc"},
                                                        }),
 
-                                                       [](const std::string& val) {
+                                                       [](std::string_view val) {
                                                          if (val != "performance")
                                                            sg_host_dvfs_plugin_init();
                                                        });
@@ -50,23 +54,14 @@ static simgrid::config::Flag<int>
     cfg_min_pstate("plugin/dvfs/min-pstate",
                    "Which pstate is the minimum (and hence fastest) pstate for this governor?", 0);
 
-static const int max_pstate_not_limited = -1;
+static constexpr int MAX_PSTATE_NOT_LIMITED = -1;
 static simgrid::config::Flag<int>
     cfg_max_pstate("plugin/dvfs/max-pstate",
-                   "Which pstate is the maximum (and hence slowest) pstate for this governor?", max_pstate_not_limited);
+                   "Which pstate is the maximum (and hence slowest) pstate for this governor?", MAX_PSTATE_NOT_LIMITED);
 
-/** @addtogroup SURF_plugin_load
+XBT_LOG_NEW_DEFAULT_SUBCATEGORY(host_dvfs, kernel, "Logging specific to the HostDvfs plugin");
 
-  This plugin makes it very simple for users to obtain the current load for each host.
-
-*/
-
-XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_plugin_dvfs, surf, "Logging specific to the SURF HostDvfs plugin");
-
-namespace simgrid {
-namespace plugin {
-
-namespace dvfs {
+namespace simgrid::plugin::dvfs {
 
 /**
  *  Add this to your host tag:
@@ -89,43 +84,39 @@ namespace dvfs {
 class Governor {
   simgrid::s4u::Host* const host_;
   double sampling_rate_;
-  int min_pstate = cfg_min_pstate; //< Never use a pstate less than this one
-  int max_pstate = cfg_max_pstate; //< Never use a pstate larger than this one
+  unsigned long min_pstate = cfg_min_pstate; //< Never use a pstate less than this one
+  unsigned long max_pstate = cfg_max_pstate; //< Never use a pstate larger than this one
 
 public:
   explicit Governor(simgrid::s4u::Host* ptr)
       : host_(ptr)
   {
-    if (cfg_max_pstate == max_pstate_not_limited)
+    if (cfg_max_pstate == MAX_PSTATE_NOT_LIMITED)
       max_pstate = host_->get_pstate_count() - 1;
     init();
   }
   virtual ~Governor() = default;
   virtual std::string get_name() const = 0;
   simgrid::s4u::Host* get_host() const { return host_; }
-  int get_min_pstate() const { return min_pstate; }
-  int get_max_pstate() const { return max_pstate; }
+  unsigned long get_min_pstate() const { return min_pstate; }
+  unsigned long get_max_pstate() const { return max_pstate; }
 
   void init()
   {
-    const char* local_sampling_rate_config = host_->get_property(cfg_sampling_rate.get_name());
-    if (local_sampling_rate_config != nullptr) {
+    if (const char* local_sampling_rate_config = host_->get_property(cfg_sampling_rate.get_name())) {
       sampling_rate_ = std::stod(local_sampling_rate_config);
     } else {
       sampling_rate_ = cfg_sampling_rate;
     }
-    const char* local_min_pstate_config = host_->get_property(cfg_min_pstate.get_name());
-    if (local_min_pstate_config != nullptr) {
-      min_pstate = std::stoi(local_min_pstate_config);
+    if (const char* local_min_pstate_config = host_->get_property(cfg_min_pstate.get_name())) {
+      min_pstate = std::stoul(local_min_pstate_config);
     }
 
-    const char* local_max_pstate_config = host_->get_property(cfg_max_pstate.get_name());
-    if (local_max_pstate_config != nullptr) {
-      max_pstate = std::stoi(local_max_pstate_config);
+    if (const char* local_max_pstate_config = host_->get_property(cfg_max_pstate.get_name())) {
+      max_pstate = std::stoul(local_max_pstate_config);
     }
     xbt_assert(max_pstate <= host_->get_pstate_count() - 1, "Value for max_pstate too large!");
     xbt_assert(min_pstate <= max_pstate, "min_pstate is larger than max_pstate!");
-    xbt_assert(0 <= min_pstate, "min_pstate is negative!");
   }
 
   virtual void update()         = 0;
@@ -195,7 +186,7 @@ public:
 
     if (load > freq_up_threshold_) {
       get_host()->set_pstate(get_min_pstate()); /* Run at max. performance! */
-      XBT_INFO("Load: %f > threshold: %f --> changed to pstate %i", load, freq_up_threshold_, get_min_pstate());
+      XBT_INFO("Load: %f > threshold: %f --> changed to pstate %lu", load, freq_up_threshold_, get_min_pstate());
     } else {
       /* The actual implementation uses a formula here: (See Kernel file cpufreq_ondemand.c:158)
        *
@@ -206,12 +197,12 @@ public:
        */
       // Load is now < freq_up_threshold; exclude pstate 0 (the fastest)
       // because pstate 0 can only be selected if load > freq_up_threshold_
-      int new_pstate = static_cast<int>(get_max_pstate() - load * (get_max_pstate() + 1));
+      auto new_pstate = get_max_pstate() - static_cast<unsigned long>(load) * (get_max_pstate() + 1);
       if (new_pstate < get_min_pstate())
         new_pstate = get_min_pstate();
       get_host()->set_pstate(new_pstate);
 
-      XBT_DEBUG("Load: %f < threshold: %f --> changed to pstate %i", load, freq_up_threshold_, new_pstate);
+      XBT_DEBUG("Load: %f < threshold: %f --> changed to pstate %lu", load, freq_up_threshold_, new_pstate);
     }
   }
 };
@@ -239,24 +230,24 @@ public:
   void update() override
   {
     double load = get_host()->get_core_count() * sg_host_get_avg_load(get_host());
-    int pstate  = get_host()->get_pstate();
+    unsigned long pstate = get_host()->get_pstate();
     sg_host_load_reset(get_host()); // Only consider the period between two calls to this method!
 
     if (load > freq_up_threshold_) {
       if (pstate != get_min_pstate()) {
         get_host()->set_pstate(pstate - 1);
-        XBT_INFO("Load: %f > threshold: %f -> increasing performance to pstate %d", load, freq_up_threshold_,
+        XBT_INFO("Load: %f > threshold: %f -> increasing performance to pstate %lu", load, freq_up_threshold_,
                  pstate - 1);
       } else {
-        XBT_DEBUG("Load: %f > threshold: %f -> but cannot speed up even more, already in highest pstate %d", load,
+        XBT_DEBUG("Load: %f > threshold: %f -> but cannot speed up even more, already in highest pstate %lu", load,
                   freq_up_threshold_, pstate);
       }
     } else if (load < freq_down_threshold_) {
       if (pstate != get_max_pstate()) { // Are we in the slowest pstate already?
         get_host()->set_pstate(pstate + 1);
-        XBT_INFO("Load: %f < threshold: %f -> slowing down to pstate %d", load, freq_down_threshold_, pstate + 1);
+        XBT_INFO("Load: %f < threshold: %f -> slowing down to pstate %lu", load, freq_down_threshold_, pstate + 1);
       } else {
-        XBT_DEBUG("Load: %f < threshold: %f -> cannot slow down even more, already in slowest pstate %d", load,
+        XBT_DEBUG("Load: %f < threshold: %f -> cannot slow down even more, already in slowest pstate %lu", load,
                   freq_down_threshold_, pstate);
       }
     }
@@ -265,11 +256,10 @@ public:
 
 #if HAVE_SMPI
 class Adagio : public Governor {
-private:
-  int best_pstate     = 0;
-  double start_time   = 0;
-  double comp_counter = 0;
-  double comp_timer   = 0;
+  unsigned long best_pstate = 0;
+  double start_time         = 0;
+  double comp_counter       = 0;
+  double comp_timer         = 0;
 
   std::vector<std::vector<double>> rates; // Each host + all frequencies of that host
 
@@ -295,21 +285,22 @@ public:
         task_id           = 0;
       }
     });
-    simgrid::s4u::Exec::on_start.connect([this](simgrid::s4u::Exec const& activity) {
+    simgrid::s4u::Exec::on_start_cb([this](simgrid::s4u::Exec const& activity) {
       if (activity.get_host() == get_host())
         pre_task();
     });
-    simgrid::s4u::Exec::on_completion.connect([this](simgrid::s4u::Exec const& activity) {
+    simgrid::s4u::Exec::on_completion_cb([this](simgrid::s4u::Exec const& exec) {
       // For more than one host (not yet supported), we can access the host via
       // simcalls_.front()->issuer->get_iface()->get_host()
-      if (activity.get_host() == get_host() && iteration_running) {
-        comp_timer += activity.get_finish_time() - activity.get_start_time();
+      if (exec.get_host() == get_host() && iteration_running) {
+        comp_timer += exec.get_finish_time() - exec.get_start_time();
       }
     });
     // FIXME I think that this fires at the same time for all hosts, so when the src sends something,
     // the dst will be notified even though it didn't even arrive at the recv yet
-    simgrid::s4u::Link::on_communicate.connect([this](const kernel::resource::NetworkAction& act) {
-      if ((get_host() == &act.get_src() || get_host() == &act.get_dst()) && iteration_running) {
+    simgrid::s4u::Comm::on_start_cb([this](const s4u::Comm& comm) {
+      if ((get_host() == comm.get_sender()->get_host() || get_host() == comm.get_receiver()->get_host()) &&
+           iteration_running) {
         post_task();
       }
     });
@@ -328,7 +319,7 @@ public:
     if (rates[task_id][best_pstate] == 0)
       best_pstate = 0;
     get_host()->set_pstate(best_pstate); // Load our schedule
-    XBT_DEBUG("Set pstate to %i", best_pstate);
+    XBT_DEBUG("Set pstate to %lu", best_pstate);
   }
 
   void post_task()
@@ -341,12 +332,12 @@ public:
     bool is_initialized         = rates[task_id][best_pstate] != 0;
     rates[task_id][best_pstate] = computed_flops / comp_timer;
     if (not is_initialized) {
-      for (int i = 1; i < get_host()->get_pstate_count(); i++) {
+      for (unsigned long i = 1; i < get_host()->get_pstate_count(); i++) {
         rates[task_id][i] = rates[task_id][0] * (get_host()->get_pstate_speed(i) / get_host()->get_speed());
       }
     }
 
-    for (int pstate = get_host()->get_pstate_count() - 1; pstate >= 0; pstate--) {
+    for (unsigned long pstate = get_host()->get_pstate_count() - 1; pstate != 0; pstate--) {
       if (computed_flops / rates[task_id][pstate] <= target_time) {
         // We just found the pstate we want to use!
         best_pstate = pstate;
@@ -359,9 +350,7 @@ public:
   void update() override {}
 };
 #endif
-} // namespace dvfs
-} // namespace plugin
-} // namespace simgrid
+} // namespace simgrid::plugin::dvfs
 
 /* **************************** events  callback *************************** */
 static void on_host_added(simgrid::s4u::Host& host)
@@ -369,7 +358,7 @@ static void on_host_added(simgrid::s4u::Host& host)
   if (dynamic_cast<simgrid::s4u::VirtualMachine*>(&host)) // Ignore virtual machines
     return;
 
-  std::string name              = std::string("dvfs-daemon-") + host.get_cname();
+  std::string name              = "dvfs-daemon-" + host.get_name();
   simgrid::s4u::ActorPtr daemon = simgrid::s4u::Actor::create(name.c_str(), &host, []() {
     /**
      * This lambda function is the function the actor (daemon) will execute
@@ -381,9 +370,8 @@ static void on_host_added(simgrid::s4u::Host& host)
     XBT_DEBUG("DVFS process on %s is a daemon: %d", daemon_proc->get_host()->get_cname(), daemon_proc->is_daemon());
 
     std::string dvfs_governor;
-    const char* host_conf = daemon_proc->get_host()->get_property("plugin/dvfs/governor");
-    if (host_conf != nullptr) {
-      dvfs_governor = std::string(host_conf);
+    if (const char* host_conf = daemon_proc->get_host()->get_property("plugin/dvfs/governor")) {
+      dvfs_governor = host_conf;
       boost::algorithm::to_lower(dvfs_governor);
     } else {
       dvfs_governor = cfg_governor;
@@ -440,5 +428,5 @@ void sg_host_dvfs_plugin_init()
 
   sg_host_load_plugin_init();
 
-  simgrid::s4u::Host::on_creation.connect(&on_host_added);
+  simgrid::s4u::Host::on_creation_cb(&on_host_added);
 }
