@@ -1,23 +1,23 @@
-/* Copyright (c) 2006-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid/Exception.hpp"
-#include "simgrid/actor.h"
-#include "simgrid/modelchecker.h"
-#include "simgrid/s4u/Actor.hpp"
-#include "simgrid/s4u/Exec.hpp"
-#include "simgrid/s4u/Host.hpp"
-#include "simgrid/s4u/VirtualMachine.hpp"
-#include "src/include/mc/mc.h"
+#include <simgrid/Exception.hpp>
+#include <simgrid/actor.h>
+#include <simgrid/modelchecker.h>
+#include <simgrid/s4u/Actor.hpp>
+#include <simgrid/s4u/Engine.hpp>
+#include <simgrid/s4u/Exec.hpp>
+#include <simgrid/s4u/VirtualMachine.hpp>
+
 #include "src/kernel/EngineImpl.hpp"
-#include "src/kernel/activity/ExecImpl.hpp"
+#include "src/kernel/actor/ActorImpl.hpp"
+#include "src/kernel/resource/HostImpl.hpp"
+#include "src/mc/mc.h"
 #include "src/mc/mc_replay.hpp"
-#include "src/surf/HostImpl.hpp"
 
 #include <algorithm>
-#include <sstream>
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_actor, s4u, "S4U actors");
 
@@ -32,10 +32,6 @@ xbt::signal<void(Actor const&)> s4u::Actor::on_suspend;
 xbt::signal<void(Actor const&)> s4u::Actor::on_resume;
 xbt::signal<void(Actor const&)> s4u::Actor::on_sleep;
 xbt::signal<void(Actor const&)> s4u::Actor::on_wake_up;
-#ifndef DOXYGEN
-xbt::signal<void(Actor const&)> s4u::Actor::on_migration_start; // XBT_ATTRIB_DEPRECATED_v329
-xbt::signal<void(Actor const&)> s4u::Actor::on_migration_end;   // XBT_ATTRIB_DEPRECATED_v329
-#endif
 xbt::signal<void(Actor const&, Host const& previous_location)> s4u::Actor::on_host_change;
 xbt::signal<void(Actor const&)> s4u::Actor::on_termination;
 xbt::signal<void(Actor const&)> s4u::Actor::on_destruction;
@@ -54,7 +50,7 @@ ActorPtr Actor::init(const std::string& name, s4u::Host* host)
 {
   const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
   kernel::actor::ActorImpl* actor =
-      kernel::actor::simcall([self, &name, host] { return self->init(name, host).get(); });
+      kernel::actor::simcall_answered([self, &name, host] { return self->init(name, host).get(); });
   return actor->get_iface();
 }
 
@@ -69,7 +65,7 @@ ActorPtr Actor::set_stacksize(unsigned stacksize)
 
 ActorPtr Actor::start(const std::function<void()>& code)
 {
-  simgrid::kernel::actor::simcall([this, &code] { pimpl_->start(code); });
+  simgrid::kernel::actor::simcall_answered([this, &code] { pimpl_->start(code); });
   return this;
 }
 
@@ -77,7 +73,7 @@ ActorPtr Actor::create(const std::string& name, s4u::Host* host, const std::func
 {
   const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
   kernel::actor::ActorImpl* actor =
-      kernel::actor::simcall([self, &name, host, &code] { return self->init(name, host)->start(code); });
+      kernel::actor::simcall_answered([self, &name, host, &code] { return self->init(name, host)->start(code); });
 
   return actor->get_iface();
 }
@@ -114,49 +110,51 @@ void Actor::join(double timeout) const
 {
   kernel::actor::ActorImpl* issuer = kernel::actor::ActorImpl::self();
   const kernel::actor::ActorImpl* target = pimpl_;
-  kernel::actor::simcall_blocking([issuer, target, timeout] {
-    if (target->finished_) {
-      // The joined actor is already finished, just wake up the issuer right away
-      issuer->simcall_answer();
-    } else {
-      kernel::activity::ActivityImplPtr sync = issuer->join(target, timeout);
-      sync->register_simcall(&issuer->simcall_);
-    }
-  });
+  kernel::actor::ActorJoinSimcall observer{issuer, get_impl(), timeout};
+
+  kernel::actor::simcall_blocking<void>(
+      [issuer, target, timeout] {
+        if (target->wannadie()) {
+          // The joined actor is already finished, just wake up the issuer right away
+          issuer->simcall_answer();
+        } else {
+          kernel::activity::ActivityImplPtr sync = issuer->join(target, timeout);
+          sync->register_simcall(&issuer->simcall_);
+        }
+      },
+      &observer);
 }
 
-void Actor::set_auto_restart(bool autorestart)
+Actor* Actor::set_auto_restart(bool autorestart)
 {
-  kernel::actor::simcall([this, autorestart]() {
-    xbt_assert(autorestart && not pimpl_->has_to_auto_restart()); // FIXME: handle all cases
+  if (autorestart == pimpl_->has_to_auto_restart()) // not changed
+    return this;
+
+  kernel::actor::simcall_answered([this, autorestart]() {
+    xbt_assert(autorestart, "Asking an actor to stop being autorestart is not implemented yet. Ask us if you need it.");
     pimpl_->set_auto_restart(autorestart);
 
     auto* arg = new kernel::actor::ProcessArg(pimpl_->get_host(), pimpl_);
     XBT_DEBUG("Adding %s to the actors_at_boot_ list of Host %s", arg->name.c_str(), arg->host->get_cname());
     pimpl_->get_host()->get_impl()->add_actor_at_boot(arg);
   });
+  return this;
+}
+int Actor::get_restart_count() const
+{
+  return pimpl_->get_restart_count();
 }
 
 void Actor::on_exit(const std::function<void(bool /*failed*/)>& fun) const
 {
-  kernel::actor::simcall([this, &fun] { pimpl_->on_exit->emplace_back(fun); });
+  kernel::actor::simcall_answered([this, &fun] { pimpl_->on_exit->emplace_back(fun); });
 }
 
 void Actor::set_host(Host* new_host)
 {
-  if (s4u::Actor::on_migration_start.get_slot_count() > 0) { // XBT_ATTRIB_DEPRECATED_v329
-    static bool already_warned = false;
-    if (not already_warned) {
-      XBT_INFO("Please use s4u::Actor::on_host_change instead of s4u::Actor::on_migration_start. This will be removed "
-               "in v3.29");
-      already_warned = true;
-    }
-    s4u::Actor::on_migration_start(*this);
-  }
-
   const s4u::Host* previous_location = get_host();
 
-  kernel::actor::simcall([this, new_host]() {
+  kernel::actor::simcall_answered([this, new_host]() {
     for (auto const& activity : pimpl_->activities_) {
       // FIXME: implement the migration of other kinds of activities
       if (auto exec = boost::dynamic_pointer_cast<kernel::activity::ExecImpl>(activity))
@@ -165,17 +163,8 @@ void Actor::set_host(Host* new_host)
     this->pimpl_->set_host(new_host);
   });
 
-  if (s4u::Actor::on_migration_end.get_slot_count() > 0) { // XBT_ATTRIB_DEPRECATED_v329
-    static bool already_warned = false;
-    if (not already_warned) {
-      XBT_INFO("Please use s4u::Actor::on_host_change instead of s4u::Actor::on_migration_end. This will be removed in "
-               "v3.29");
-      already_warned = true;
-    }
-    s4u::Actor::on_migration_end(*this);
-  }
-
   s4u::Actor::on_host_change(*this, *previous_location);
+  s4u::Actor::on_this_host_change(*this, *previous_location);
 }
 
 s4u::Host* Actor::get_host() const
@@ -183,9 +172,10 @@ s4u::Host* Actor::get_host() const
   return this->pimpl_->get_host();
 }
 
-void Actor::daemonize()
+Actor* Actor::daemonize()
 {
-  kernel::actor::simcall([this]() { pimpl_->daemonize(); });
+  kernel::actor::simcall_answered([this]() { pimpl_->daemonize(); });
+  return this;
 }
 
 bool Actor::is_daemon() const
@@ -193,7 +183,13 @@ bool Actor::is_daemon() const
   return this->pimpl_->is_daemon();
 }
 
-const simgrid::xbt::string& Actor::get_name() const
+bool Actor::is_maestro()
+{
+  const auto* self = kernel::actor::ActorImpl::self();
+  return self == nullptr || kernel::EngineImpl::get_instance()->is_maestro(self);
+}
+
+const std::string& Actor::get_name() const
 {
   return this->pimpl_->get_name();
 }
@@ -218,19 +214,24 @@ void Actor::suspend()
   kernel::actor::ActorImpl* issuer = kernel::actor::ActorImpl::self();
   kernel::actor::ActorImpl* target = pimpl_;
   s4u::Actor::on_suspend(*this);
-  kernel::actor::simcall_blocking([issuer, target]() {
-    target->suspend();
-    if (target != issuer) {
-      /* If we are suspending ourselves, then just do not finish the simcall now */
-      issuer->simcall_answer();
-    }
-  });
+  s4u::Actor::on_this_suspend(*this);
+  kernel::actor::ActorSuspendSimcall observer(issuer);
+  kernel::actor::simcall_blocking<void>(
+      [issuer, target]() {
+        target->suspend();
+        if (target != issuer) {
+          /* If we are suspending ourselves, then just do not finish the simcall now */
+          issuer->simcall_answer();
+        }
+      },
+      &observer);
 }
 
 void Actor::resume()
 {
-  kernel::actor::simcall([this] { pimpl_->resume(); });
+  kernel::actor::simcall_answered([this] { pimpl_->resume(); });
   s4u::Actor::on_resume(*this);
+  s4u::Actor::on_this_resume(*this);
 }
 
 bool Actor::is_suspended() const
@@ -240,7 +241,7 @@ bool Actor::is_suspended() const
 
 void Actor::set_kill_time(double kill_time)
 {
-  kernel::actor::simcall([this, kill_time] { pimpl_->set_kill_time(kill_time); });
+  kernel::actor::simcall_answered([this, kill_time] { pimpl_->set_kill_time(kill_time); });
 }
 
 /** @brief Get the kill time of an actor(or 0 if unset). */
@@ -252,14 +253,14 @@ double Actor::get_kill_time() const
 void Actor::kill()
 {
   const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
-  kernel::actor::simcall([this, self] { self->kill(pimpl_); });
+  kernel::actor::simcall_answered([this, self] { self->kill(pimpl_); });
 }
 
 // ***** Static functions *****
 
 ActorPtr Actor::by_pid(aid_t pid)
 {
-  kernel::actor::ActorImpl* actor = kernel::actor::ActorImpl::by_pid(pid);
+  kernel::actor::ActorImpl* actor = kernel::EngineImpl::get_instance()->get_actor_by_pid(pid);
   if (actor != nullptr)
     return actor->get_iface();
   else
@@ -269,7 +270,7 @@ ActorPtr Actor::by_pid(aid_t pid)
 void Actor::kill_all()
 {
   const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
-  kernel::actor::simcall([self] { self->kill_all(); });
+  kernel::actor::simcall_answered([self] { self->kill_all(); });
 }
 
 const std::unordered_map<std::string, std::string>* Actor::get_properties() const
@@ -285,12 +286,12 @@ const char* Actor::get_property(const std::string& key) const
 
 void Actor::set_property(const std::string& key, const std::string& value)
 {
-  kernel::actor::simcall([this, &key, &value] { pimpl_->set_property(key, value); });
+  kernel::actor::simcall_answered([this, &key, &value] { pimpl_->set_property(key, value); });
 }
 
 Actor* Actor::restart()
 {
-  return kernel::actor::simcall([this]() { return pimpl_->restart(); });
+  return kernel::actor::simcall_answered([this]() { return pimpl_->restart(); });
 }
 
 // ***** this_actor *****
@@ -308,7 +309,7 @@ namespace this_actor {
  */
 bool is_maestro()
 {
-  return SIMIX_is_maestro();
+  return Actor::is_maestro();
 }
 
 void sleep_for(double duration)
@@ -318,36 +319,42 @@ void sleep_for(double duration)
   if (duration <= 0) /* that's a no-op */
     return;
 
-  if (duration < sg_surf_precision) {
+  if (duration < sg_precision_timing) {
     static unsigned int warned = 0; // At most 20 such warnings
     warned++;
     if (warned <= 20)
       XBT_INFO("The parameter to sleep_for() is smaller than the SimGrid numerical accuracy (%g < %g). "
                "Please refer to https://simgrid.org/doc/latest/Configuring_SimGrid.html#numerical-precision",
-               duration, sg_surf_precision);
+               duration, sg_precision_timing);
     if (warned == 20)
       XBT_VERB("(further warnings about the numerical accuracy of sleep_for() will be omitted).");
   }
 
   kernel::actor::ActorImpl* issuer = kernel::actor::ActorImpl::self();
-  Actor::on_sleep(*issuer->get_ciface());
+  kernel::actor::ActorSleepSimcall observer(issuer);
 
-  kernel::actor::simcall_blocking([issuer, duration]() {
-    if (MC_is_active() || MC_record_replay_is_active()) {
-      MC_process_clock_add(issuer, duration);
-      issuer->simcall_answer();
-      return;
-    }
-    kernel::activity::ActivityImplPtr sync = issuer->sleep(duration);
-    sync->register_simcall(&issuer->simcall_);
-  });
+  Actor::on_sleep(*issuer->get_ciface());
+  issuer->get_ciface()->on_this_sleep(*issuer->get_ciface());
+
+  kernel::actor::simcall_blocking<void>(
+      [issuer, duration]() {
+        if (MC_is_active() || MC_record_replay_is_active()) {
+          // MC_process_clock_add(issuer, duration); // BUG: Makes the exploration loop
+          issuer->simcall_answer();
+        } else {
+          kernel::activity::ActivityImplPtr sync = issuer->sleep(duration);
+          sync->register_simcall(&issuer->simcall_);
+        }
+      },
+      &observer);
 
   Actor::on_wake_up(*issuer->get_ciface());
+  issuer->get_ciface()->on_this_wake_up(*issuer->get_ciface());
 }
 
 void yield()
 {
-  kernel::actor::simcall([] { /* do nothing*/ });
+  kernel::actor::simcall_answered([] { /* do nothing*/ });
 }
 
 XBT_PUBLIC void sleep_until(double wakeup_time)
@@ -364,7 +371,7 @@ void execute(double flops)
 
 void execute(double flops, double priority)
 {
-  exec_init(flops)->set_priority(priority)->vetoable_start()->wait();
+  exec_init(flops)->set_priority(priority)->start()->wait();
 }
 
 void parallel_execute(const std::vector<s4u::Host*>& hosts, const std::vector<double>& flops_amounts,
@@ -373,10 +380,9 @@ void parallel_execute(const std::vector<s4u::Host*>& hosts, const std::vector<do
   exec_init(hosts, flops_amounts, bytes_amounts)->wait();
 }
 
-void parallel_execute(const std::vector<s4u::Host*>& hosts, const std::vector<double>& flops_amounts,
-                      const std::vector<double>& bytes_amounts, double timeout) // XBT_ATTRIB_DEPRECATED_v329
+void thread_execute(s4u::Host* host, double flops_amount, int thread_count)
 {
-  exec_init(hosts, flops_amounts, bytes_amounts)->wait_for(timeout);
+  Exec::init()->set_flops_amount(flops_amount)->set_host(host)->set_thread_count(thread_count)->wait();
 }
 
 ExecPtr exec_init(double flops_amount)
@@ -413,13 +419,14 @@ ExecPtr exec_init(const std::vector<s4u::Host*>& hosts, const std::vector<double
 ExecPtr exec_async(double flops)
 {
   ExecPtr res = exec_init(flops);
-  res->vetoable_start();
+  res->start();
   return res;
 }
 
 aid_t get_pid()
 {
-  return simgrid::kernel::actor::ActorImpl::self()->get_pid();
+  const auto* self = simgrid::kernel::actor::ActorImpl::self();
+  return self ? self->get_pid() : 0;
 }
 
 aid_t get_ppid()
@@ -434,7 +441,8 @@ std::string get_name()
 
 const char* get_cname()
 {
-  return simgrid::kernel::actor::ActorImpl::self()->get_cname();
+  const auto* self = simgrid::kernel::actor::ActorImpl::self();
+  return self ? self->get_cname() : nullptr;
 }
 
 Host* get_host()
@@ -445,14 +453,14 @@ Host* get_host()
 void suspend()
 {
   kernel::actor::ActorImpl* self = simgrid::kernel::actor::ActorImpl::self();
-  s4u::Actor::on_suspend(*self->get_ciface());
-  kernel::actor::simcall_blocking([self] { self->suspend(); });
+  self->get_iface()->suspend();
 }
 
 void exit()
 {
   kernel::actor::ActorImpl* self = simgrid::kernel::actor::ActorImpl::self();
-  simgrid::kernel::actor::simcall([self] { self->exit(); });
+  simgrid::kernel::actor::simcall_answered([self] { self->exit(); });
+  THROW_IMPOSSIBLE;
 }
 
 void on_exit(const std::function<void(bool)>& fun)
@@ -468,10 +476,6 @@ void set_host(Host* new_host)
 {
   simgrid::kernel::actor::ActorImpl::self()->get_iface()->set_host(new_host);
 }
-void migrate(Host* new_host) // XBT_ATTRIB_DEPRECATED_v329
-{
-  set_host(new_host);
-}
 
 } // namespace this_actor
 } // namespace s4u
@@ -509,6 +513,12 @@ void sg_actor_start_(sg_actor_t actor, xbt_main_func_t code, int argc, const cha
   actor->start(function);
 }
 
+void sg_actor_start_voidp(sg_actor_t actor, void_f_pvoid_t code, void* param)
+{
+  simgrid::kernel::actor::ActorCode function;
+  actor->start(code, param);
+}
+
 sg_actor_t sg_actor_create_(const char* name, sg_host_t host, xbt_main_func_t code, int argc, const char* const* argv)
 {
   simgrid::kernel::actor::ActorCode function = simgrid::xbt::wrap_main(code, argc, argv);
@@ -563,21 +573,6 @@ sg_actor_t sg_actor_by_pid(aid_t pid)
   return simgrid::s4u::Actor::by_pid(pid).get();
 }
 
-aid_t sg_actor_get_PID(const_sg_actor_t actor) // XBT_ATTRIB_DEPRECATED_v331
-{
-  return sg_actor_get_pid(actor);
-}
-
-aid_t sg_actor_get_PPID(const_sg_actor_t actor) // XBT_ATTRIB_DEPRECATED_v331
-{
-  return sg_actor_get_ppid(actor);
-}
-
-sg_actor_t sg_actor_by_PID(aid_t pid) // XBT_ATTRIB_DEPRECATED_v331
-{
-  return sg_actor_by_pid(pid);
-}
-
 /** @brief Return the name of an actor. */
 const char* sg_actor_get_name(const_sg_actor_t actor)
 {
@@ -613,8 +608,8 @@ xbt_dict_t sg_actor_get_properties(const_sg_actor_t actor)
   const std::unordered_map<std::string, std::string>* props = actor->get_properties();
   if (props == nullptr)
     return nullptr;
-  for (auto const& kv : *props) {
-    xbt_dict_set(as_dict, kv.first.c_str(), xbt_strdup(kv.second.c_str()));
+  for (auto const& [key, value] : *props) {
+    xbt_dict_set(as_dict, key.c_str(), xbt_strdup(value.c_str()));
   }
   return as_dict;
 }
@@ -687,10 +682,6 @@ void sg_actor_set_host(sg_actor_t actor, sg_host_t host)
 {
   actor->set_host(host);
 }
-void sg_actor_migrate(sg_actor_t actor, sg_host_t host) // XBT_ATTRIB_DEPRECATED_v329
-{
-  actor->set_host(host);
-}
 
 /**
  * @brief Wait for the completion of a #sg_actor_t.
@@ -752,7 +743,7 @@ sg_actor_t sg_actor_attach(const char* name, void* data, sg_host_t host, xbt_dic
   xbt_dict_free(&properties);
 
   /* Let's create the actor: SIMIX may decide to start it right now, even before returning the flow control to us */
-  smx_actor_t actor = nullptr;
+  simgrid::kernel::actor::ActorImpl* actor = nullptr;
   try {
     actor = simgrid::kernel::actor::ActorImpl::attach(name, data, host).get();
     actor->set_properties(props);
@@ -781,12 +772,14 @@ aid_t sg_actor_self_get_ppid()
 
 const char* sg_actor_self_get_name()
 {
+  if (simgrid::s4u::Actor::is_maestro())
+    return "maestro";
   return simgrid::s4u::this_actor::get_cname();
 }
 
 void* sg_actor_self_get_data()
 {
-  return simgrid::s4u::Actor::self()->get_data();
+  return simgrid::s4u::Actor::self()->get_data<void>();
 }
 
 void sg_actor_self_set_data(void* userdata)
@@ -794,26 +787,11 @@ void sg_actor_self_set_data(void* userdata)
   simgrid::s4u::Actor::self()->set_data(userdata);
 }
 
-void* sg_actor_self_data() // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_actor_self_get_data();
-}
-
-void sg_actor_self_data_set(void* userdata) // XBT_ATTRIB_DEPRECATED_v330
-{
-  sg_actor_self_set_data(userdata);
-}
-
 sg_actor_t sg_actor_self()
 {
   return simgrid::s4u::Actor::self();
 }
 
-void sg_actor_self_execute(double flops) // XBT_ATTRIB_DEPRECATED_v330
-{
-  simgrid::s4u::this_actor::execute(flops);
-}
-
 void sg_actor_execute(double flops)
 {
   simgrid::s4u::this_actor::execute(flops);
@@ -850,7 +828,7 @@ void sg_actor_unref(const_sg_actor_t actor)
 /** @brief Return the user data of a #sg_actor_t */
 void* sg_actor_get_data(const_sg_actor_t actor)
 {
-  return actor->get_data();
+  return actor->get_data<void>();
 }
 
 /** @brief Set the user data of a #sg_actor_t */
@@ -859,16 +837,6 @@ void sg_actor_set_data(sg_actor_t actor, void* userdata)
   actor->set_data(userdata);
 }
 
-void* sg_actor_data(const_sg_actor_t actor) // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_actor_get_data(actor);
-}
-
-void sg_actor_data_set(sg_actor_t actor, void* userdata) // XBT_ATTRIB_DEPRECATED_v330
-{
-  sg_actor_set_data(actor, userdata);
-}
-
 /** @brief Add a function to the list of "on_exit" functions for the current actor.
  *  The on_exit functions are the functions executed when your actor is killed.
  *  You should use them to free the data used by your actor.
