@@ -1,10 +1,12 @@
-/* Copyright (c) 2006-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
+#include "simgrid/simcall.hpp"
 #include <simgrid/Exception.hpp>
 #include <simgrid/exec.h>
+#include <simgrid/s4u/ActivitySet.hpp>
 #include <simgrid/s4u/Exec.hpp>
 #include <simgrid/s4u/Host.hpp>
 
@@ -14,108 +16,99 @@
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_exec, s4u_activity, "S4U asynchronous executions");
 
-namespace simgrid {
-namespace s4u {
-xbt::signal<void(Exec const&)> Exec::on_start;
-xbt::signal<void(Exec const&)> Exec::on_completion;
+namespace simgrid::s4u {
 
 Exec::Exec(kernel::activity::ExecImplPtr pimpl)
 {
   pimpl_ = pimpl;
 }
 
-void Exec::complete(Activity::State state)
+void Exec::reset() const
 {
-  Activity::complete(state);
-  on_completion(*this);
+  boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->reset();
 }
 
 ExecPtr Exec::init()
 {
   auto pimpl = kernel::activity::ExecImplPtr(new kernel::activity::ExecImpl());
-  unsigned int cb_id = Host::on_state_change.connect([pimpl](s4u::Host const& h) {
-    if (not h.is_on() && pimpl->state_ == kernel::activity::State::RUNNING &&
-        std::find(pimpl->get_hosts().begin(), pimpl->get_hosts().end(), &h) != pimpl->get_hosts().end()) {
-      pimpl->state_ = kernel::activity::State::FAILED;
-      pimpl->post();
-    }
-  });
-  pimpl->set_cb_id(cb_id);
-  return ExecPtr(pimpl->get_iface());
+  return ExecPtr(static_cast<Exec*>(pimpl->get_iface()));
 }
 
-Exec* Exec::start()
+Exec* Exec::do_start()
 {
-  kernel::actor::simcall([this] {
-    (*boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_))
-        .set_name(get_name())
-        .set_tracing_category(get_tracing_category())
-        .start();
+  kernel::actor::simcall_answered([this] {
+    auto pimpl = boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_);
+    pimpl->set_name(get_name());
+    pimpl->set_tracing_category(get_tracing_category());
+    if (detached_)
+      pimpl->detach();
+    pimpl->start();
   });
 
   if (suspended_)
     pimpl_->suspend();
 
   state_      = State::STARTED;
-  on_start(*this);
+  fire_on_start();
+  fire_on_this_start();
   return this;
 }
 
-ssize_t Exec::wait_any_for(const std::vector<ExecPtr>& execs, double timeout)
+ssize_t Exec::deprecated_wait_any_for(const std::vector<ExecPtr>& execs, double timeout) // XBT_ATTRIB_DEPRECATED_v339
 {
-  std::vector<kernel::activity::ExecImpl*> rexecs(execs.size());
-  std::transform(begin(execs), end(execs), begin(rexecs),
-                 [](const ExecPtr& exec) { return static_cast<kernel::activity::ExecImpl*>(exec->pimpl_.get()); });
+  if (execs.empty())
+    return -1;
+  ActivitySet set;
+  for (const auto& exec : execs)
+    set.push(exec);
+  try {
+    auto* ret = set.wait_any_for(timeout).get();
+    for (size_t i = 0; i < execs.size(); i++)
+      if (execs[i].get() == ret)
+        return i;
 
-  kernel::actor::ActorImpl* issuer = kernel::actor::ActorImpl::self();
-  kernel::actor::ExecutionWaitanySimcall observer{issuer, rexecs, timeout};
-  ssize_t changed_pos = kernel::actor::simcall_blocking(
-      [&observer] {
-        kernel::activity::ExecImpl::wait_any_for(observer.get_issuer(), observer.get_execs(), observer.get_timeout());
-      },
-      &observer);
-  if (changed_pos != -1)
-    execs.at(changed_pos)->complete(State::FINISHED);
-  return changed_pos;
+  } catch (TimeoutException& e) {
+    return -1;
+  }
+  return -1;
 }
 
-/** @brief change the execution bound
- * This means changing the maximal amount of flops per second that it may consume, regardless of what the host may
- * deliver. Currently, this cannot be changed once the exec started.
- */
 ExecPtr Exec::set_bound(double bound)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
              "Cannot change the bound of an exec after its start");
-  kernel::actor::simcall(
-      [this, bound] { boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_bound(bound); });
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, bound] {
+    boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_bound(bound);
+  });
   return this;
 }
 
-/** @brief  Change the execution priority, don't you think?
- *
- * An execution with twice the priority will get twice the amount of flops when the resource is shared.
- * The default priority is 1.
- *
- * Currently, this cannot be changed once the exec started. */
 ExecPtr Exec::set_priority(double priority)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
              "Cannot change the priority of an exec after its start");
-  kernel::actor::simcall([this, priority] {
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, priority] {
     boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_sharing_penalty(1. / priority);
   });
   return this;
 }
 
+ExecPtr Exec::update_priority(double priority)
+{
+  kernel::actor::simcall_answered([this, priority] {
+    boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->update_sharing_penalty(1. / priority);
+  });
+  return this;
+}
+
 ExecPtr Exec::set_flops_amount(double flops_amount)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
       "Cannot change the flop_amount of an exec after its start");
-  kernel::actor::simcall([this, flops_amount] {
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, flops_amount] {
     boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_flops_amount(flops_amount);
   });
-  Activity::set_remaining(flops_amount);
+  set_remaining(flops_amount);
   return this;
 }
 
@@ -123,7 +116,7 @@ ExecPtr Exec::set_flops_amounts(const std::vector<double>& flops_amounts)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
       "Cannot change the flops_amounts of an exec after its start");
-  kernel::actor::simcall([this, flops_amounts] {
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, flops_amounts] {
     boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_flops_amounts(flops_amounts);
   });
   parallel_      = true;
@@ -134,16 +127,23 @@ ExecPtr Exec::set_bytes_amounts(const std::vector<double>& bytes_amounts)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
       "Cannot change the bytes_amounts of an exec after its start");
-  kernel::actor::simcall([this, bytes_amounts] {
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, bytes_amounts] {
     boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_bytes_amounts(bytes_amounts);
   });
   parallel_      = true;
   return this;
 }
 
-/** @brief Retrieve the host on which this activity takes place.
- *  If it runs on more than one host, only the first host is returned.
- */
+ExecPtr Exec::set_thread_count(int thread_count)
+{
+  xbt_assert(state_ == State::INITED || state_ == State::STARTING,
+             "Cannot change the bytes_amounts of an exec after its start");
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, thread_count] {
+    boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_thread_count(thread_count);
+  });
+  return this;
+}
+
 Host* Exec::get_host() const
 {
   return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_host();
@@ -153,19 +153,11 @@ unsigned int Exec::get_host_number() const
   return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_host_number();
 }
 
-double Exec::get_start_time() const
+int Exec::get_thread_count() const
 {
-  return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_start_time();
+  return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_thread_count();
 }
 
-double Exec::get_finish_time() const
-{
-  return static_cast<kernel::activity::ExecImpl*>(pimpl_.get())->get_finish_time();
-}
-
-/** @brief Change the host on which this activity takes place.
- *
- * The activity cannot be terminated already (but it may be started). */
 ExecPtr Exec::set_host(Host* host)
 {
   xbt_assert(state_ == State::INITED || state_ == State::STARTING || state_ == State::STARTED,
@@ -174,12 +166,12 @@ ExecPtr Exec::set_host(Host* host)
   if (state_ == State::STARTED)
     boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->migrate(host);
 
-  kernel::actor::simcall(
-      [this, host] { boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_host(host); });
+  kernel::actor::simcall_object_access(
+      pimpl_.get(), [this, host] { boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_host(host); });
 
   if (state_ == State::STARTING)
-  // Setting the host may allow to start the activity, let's try
-    vetoable_start();
+    // Setting the host may allow to start the activity, let's try
+    start();
 
   return this;
 }
@@ -189,20 +181,37 @@ ExecPtr Exec::set_hosts(const std::vector<Host*>& hosts)
   xbt_assert(state_ == State::INITED || state_ == State::STARTING,
              "Cannot change the hosts of an exec once it's done (state: %s)", to_c_str(state_));
 
-  kernel::actor::simcall(
-      [this, hosts] { boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_hosts(hosts); });
+  kernel::actor::simcall_object_access(pimpl_.get(), [this, hosts] {
+    boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->set_hosts(hosts);
+  });
   parallel_ = true;
 
   // Setting the host may allow to start the activity, let's try
   if (state_ == State::STARTING)
-     vetoable_start();
+     start();
 
   return this;
 }
 
+ExecPtr Exec::unset_host()
+{
+  if (not is_assigned())
+    throw std::invalid_argument(
+        xbt::string_printf("Exec %s: the activity is not assigned to any host(s)", get_cname()));
+  else {
+    reset();
+
+    if (state_ == State::STARTED)
+      cancel();
+    start();
+
+    return this;
+  }
+}
+
 double Exec::get_cost() const
 {
-  return (pimpl_->surf_action_ == nullptr) ? -1 : pimpl_->surf_action_->get_cost();
+  return (pimpl_->model_action_ == nullptr) ? -1 : pimpl_->model_action_->get_cost();
 }
 
 double Exec::get_remaining() const
@@ -211,21 +220,17 @@ double Exec::get_remaining() const
     XBT_WARN("Calling get_remaining() on a parallel execution is not allowed. Call get_remaining_ratio() instead.");
     return get_remaining_ratio();
   } else
-    return kernel::actor::simcall(
+    return kernel::actor::simcall_answered(
         [this]() { return boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->get_remaining(); });
 }
 
-/** @brief Returns the ratio of elements that are still to do
- *
- * The returned value is between 0 (completely done) and 1 (nothing done yet).
- */
 double Exec::get_remaining_ratio() const
 {
   if (is_parallel())
-    return kernel::actor::simcall(
+    return kernel::actor::simcall_answered(
         [this]() { return boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->get_par_remaining_ratio(); });
   else
-    return kernel::actor::simcall(
+    return kernel::actor::simcall_answered(
         [this]() { return boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->get_seq_remaining_ratio(); });
 }
 
@@ -233,10 +238,14 @@ bool Exec::is_assigned() const
 {
   return not boost::static_pointer_cast<kernel::activity::ExecImpl>(pimpl_)->get_hosts().empty();
 }
-} // namespace s4u
-} // namespace simgrid
+} // namespace simgrid::s4u
 
 /* **************************** Public C interface *************************** */
+int sg_exec_isinstance(sg_activity_t acti)
+{
+  return dynamic_cast<simgrid::s4u::Exec*>(acti) != nullptr;
+}
+
 void sg_exec_set_bound(sg_exec_t exec, double bound)
 {
   exec->set_bound(bound);
@@ -269,7 +278,7 @@ double sg_exec_get_remaining_ratio(const_sg_exec_t exec)
 
 void sg_exec_start(sg_exec_t exec)
 {
-  exec->vetoable_start();
+  exec->start();
 }
 
 void sg_exec_cancel(sg_exec_t exec)
@@ -309,18 +318,27 @@ sg_error_t sg_exec_wait_for(sg_exec_t exec, double timeout)
   return status;
 }
 
-ssize_t sg_exec_wait_any(sg_exec_t* execs, size_t count)
-{
-  return sg_exec_wait_any_for(execs, count, -1.0);
-}
-
-ssize_t sg_exec_wait_any_for(sg_exec_t* execs, size_t count, double timeout)
+ssize_t sg_exec_wait_any(sg_exec_t* execs, size_t count) // XBT_ATTRIB_DEPRECATED_v339
 {
   std::vector<simgrid::s4u::ExecPtr> s4u_execs;
   for (size_t i = 0; i < count; i++)
     s4u_execs.emplace_back(execs[i], false);
 
-  ssize_t pos = simgrid::s4u::Exec::wait_any_for(s4u_execs, timeout);
+  ssize_t pos = simgrid::s4u::Exec::deprecated_wait_any_for(s4u_execs, -1.0);
+  for (size_t i = 0; i < count; i++) {
+    if (pos != -1 && static_cast<size_t>(pos) != i)
+      s4u_execs[i]->add_ref();
+  }
+  return pos;
+}
+
+ssize_t sg_exec_wait_any_for(sg_exec_t* execs, size_t count, double timeout) // XBT_ATTRIB_DEPRECATED_v339
+{
+  std::vector<simgrid::s4u::ExecPtr> s4u_execs;
+  for (size_t i = 0; i < count; i++)
+    s4u_execs.emplace_back(execs[i], false);
+
+  ssize_t pos = simgrid::s4u::Exec::deprecated_wait_any_for(s4u_execs, timeout);
   for (size_t i = 0; i < count; i++) {
     if (pos != -1 && static_cast<size_t>(pos) != i)
       s4u_execs[i]->add_ref();
