@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2006-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -13,13 +13,15 @@
 #include <simgrid/s4u/VirtualMachine.hpp>
 #include <xbt/parse_units.hpp>
 
-#include "src/plugins/vm/VirtualMachineImpl.hpp"
-#include "src/surf/HostImpl.hpp"
+#include "simgrid/simcall.hpp"
+#include "src/kernel/resource/HostImpl.hpp"
+#include "src/kernel/resource/StandardLinkImpl.hpp"
+#include "src/kernel/resource/VirtualMachineImpl.hpp"
 
 #include <string>
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(s4u_host, s4u, "Logging specific to the S4U hosts");
-XBT_LOG_EXTERNAL_CATEGORY(surf_route);
+XBT_LOG_EXTERNAL_CATEGORY(ker_platform);
 
 namespace simgrid {
 
@@ -27,10 +29,13 @@ template class xbt::Extendable<s4u::Host>;
 
 namespace s4u {
 
+#ifndef DOXYGEN
 xbt::signal<void(Host&)> Host::on_creation;
 xbt::signal<void(Host const&)> Host::on_destruction;
-xbt::signal<void(Host const&)> Host::on_state_change;
+xbt::signal<void(Host const&)> Host::on_onoff;
 xbt::signal<void(Host const&)> Host::on_speed_change;
+xbt::signal<void(kernel::resource::CpuAction&, kernel::resource::Action::State)> Host::on_exec_state_change;
+#endif
 
 Host* Host::set_cpu(kernel::resource::CpuImpl* cpu)
 {
@@ -62,7 +67,7 @@ Host::~Host()
  */
 void Host::destroy()
 {
-  kernel::actor::simcall([this] { this->pimpl_->destroy(); });
+  kernel::actor::simcall_answered([this] { this->pimpl_->destroy(); });
 }
 
 Host* Host::by_name(const std::string& name)
@@ -76,12 +81,12 @@ Host* Host::by_name_or_null(const std::string& name)
 
 Host* Host::current()
 {
-  kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
+  const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
   xbt_assert(self != nullptr, "Cannot call Host::current() from the maestro context");
   return self->get_host();
 }
 
-xbt::string const& Host::get_name() const
+std::string const& Host::get_name() const
 {
   return this->pimpl_->get_name();
 }
@@ -94,10 +99,11 @@ const char* Host::get_cname() const
 void Host::turn_on()
 {
   if (not is_on()) {
-    kernel::actor::simcall([this] {
+    kernel::actor::simcall_answered([this] {
       this->pimpl_cpu_->turn_on();
       this->pimpl_->turn_on();
-      on_state_change(*this);
+      on_onoff(*this);
+      on_this_onoff(*this);
     });
   }
 }
@@ -107,16 +113,12 @@ void Host::turn_off()
 {
   if (is_on()) {
     const kernel::actor::ActorImpl* self = kernel::actor::ActorImpl::self();
-    kernel::actor::simcall([this, self] {
-      for (VirtualMachine* const& vm : vm::VirtualMachineImpl::allVms_)
-        if (vm->get_pm() == this) {
-          vm->shutdown();
-          vm->turn_off();
-        }
+    kernel::actor::simcall_answered([this, self] {
       this->pimpl_cpu_->turn_off();
       this->pimpl_->turn_off(self);
 
-      on_state_change(*this);
+      on_onoff(*this);
+      on_this_onoff(*this);
     });
   }
 }
@@ -160,21 +162,31 @@ size_t Host::get_actor_count() const
  */
 void Host::route_to(const Host* dest, std::vector<Link*>& links, double* latency) const
 {
-  std::vector<kernel::resource::LinkImpl*> linkImpls;
+  std::vector<kernel::resource::StandardLinkImpl*> linkImpls;
   this->route_to(dest, linkImpls, latency);
   for (auto* l : linkImpls)
     links.push_back(l->get_iface());
 }
+std::pair<std::vector<Link*>, double> Host::route_to(const Host* dest) const
+{
+  std::vector<kernel::resource::StandardLinkImpl*> linkImpls;
+  std::vector<Link*> links;
+  double latency = 0;
+  this->route_to(dest, linkImpls, &latency);
+  for (auto* l : linkImpls)
+    links.push_back(l->get_iface());
+  return std::make_pair(links, latency);
+}
 
 /** @brief Just like Host::routeTo, but filling an array of link implementations */
-void Host::route_to(const Host* dest, std::vector<kernel::resource::LinkImpl*>& links, double* latency) const
+void Host::route_to(const Host* dest, std::vector<kernel::resource::StandardLinkImpl*>& links, double* latency) const
 {
   kernel::routing::NetZoneImpl::get_global_route(pimpl_netpoint_, dest->get_netpoint(), links, latency);
-  if (XBT_LOG_ISENABLED(surf_route, xbt_log_priority_debug)) {
-    XBT_CDEBUG(surf_route, "Route from '%s' to '%s' (latency: %f):", get_cname(), dest->get_cname(),
+  if (XBT_LOG_ISENABLED(ker_platform, xbt_log_priority_debug)) {
+    XBT_CDEBUG(ker_platform, "Route from '%s' to '%s' (latency: %f):", get_cname(), dest->get_cname(),
                (latency == nullptr ? -1 : *latency));
     for (auto const* link : links)
-      XBT_CDEBUG(surf_route, "  Link '%s'", link->get_cname());
+      XBT_CDEBUG(ker_platform, "  Link '%s'", link->get_cname());
   }
 }
 
@@ -184,23 +196,6 @@ NetZone* Host::get_englobing_zone() const
   return pimpl_netpoint_->get_englobing_zone()->get_iface();
 }
 
-#ifndef DOXYGEN
-void Host::sendto(Host* dest, double byte_amount) // XBT_ATTRIB_DEPRECATED_v331
-{
-  Comm::sendto_async(this, dest, byte_amount)->wait();
-}
-
-CommPtr Host::sendto_async(Host* dest, double byte_amount) // XBT_ATTRIB_DEPRECATED_v331
-{
-  return Comm::sendto_async(this, dest, byte_amount);
-}
-
-void Host::send_to(Host* dest, double byte_amount) // XBT_ATTRIB_DEPRECATED_v330
-{
-  Comm::sendto(this, dest, byte_amount);
-}
-#endif
-
 /** Get the properties assigned to a host */
 const std::unordered_map<std::string, std::string>* Host::get_properties() const
 {
@@ -215,13 +210,24 @@ const char* Host::get_property(const std::string& key) const
 
 Host* Host::set_property(const std::string& key, const std::string& value)
 {
-  kernel::actor::simcall([this, &key, &value] { this->pimpl_->set_property(key, value); });
+  kernel::actor::simcall_object_access(pimpl_, [this, &key, &value] { this->pimpl_->set_property(key, value); });
   return this;
 }
 
 Host* Host::set_properties(const std::unordered_map<std::string, std::string>& properties)
 {
-  kernel::actor::simcall([this, &properties] { this->pimpl_->set_properties(properties); });
+  kernel::actor::simcall_object_access(pimpl_, [this, &properties] { this->pimpl_->set_properties(properties); });
+  return this;
+}
+
+int Host::get_concurrency_limit() const
+{
+  return pimpl_cpu_->get_concurrency_limit();
+}
+
+Host* Host::set_concurrency_limit(int limit)
+{
+  kernel::actor::simcall_object_access(pimpl_cpu_, [this, limit] { pimpl_cpu_->set_concurrency_limit(limit); });
   return this;
 }
 
@@ -229,7 +235,7 @@ Host* Host::set_properties(const std::unordered_map<std::string, std::string>& p
  * The profile must contain boolean values. */
 Host* Host::set_state_profile(kernel::profile::Profile* p)
 {
-  kernel::actor::simcall([this, p] { pimpl_cpu_->set_state_profile(p); });
+  kernel::actor::simcall_object_access(pimpl_, [this, p] { pimpl_cpu_->set_state_profile(p); });
   return this;
 }
 /** Specify a profile modeling the external load according to an exhaustive list or a stochastic law.
@@ -240,7 +246,7 @@ Host* Host::set_state_profile(kernel::profile::Profile* p)
  */
 Host* Host::set_speed_profile(kernel::profile::Profile* p)
 {
-  kernel::actor::simcall([this, p] { pimpl_cpu_->set_speed_profile(p); });
+  kernel::actor::simcall_object_access(pimpl_, [this, p] { pimpl_cpu_->set_speed_profile(p); });
   return this;
 }
 
@@ -265,7 +271,7 @@ double Host::get_available_speed() const
 
 Host* Host::set_sharing_policy(SharingPolicy policy, const s4u::NonLinearResourceCb& cb)
 {
-  kernel::actor::simcall([this, policy, &cb] { pimpl_cpu_->set_sharing_policy(policy, cb); });
+  kernel::actor::simcall_object_access(pimpl_, [this, policy, &cb] { pimpl_cpu_->set_sharing_policy(policy, cb); });
   return this;
 }
 
@@ -281,13 +287,14 @@ int Host::get_core_count() const
 
 Host* Host::set_core_count(int core_count)
 {
-  kernel::actor::simcall([this, core_count] { this->pimpl_cpu_->set_core_count(core_count); });
+  kernel::actor::simcall_object_access(pimpl_, [this, core_count] { this->pimpl_cpu_->set_core_count(core_count); });
   return this;
 }
 
 Host* Host::set_pstate_speed(const std::vector<double>& speed_per_state)
 {
-  kernel::actor::simcall([this, &speed_per_state] { pimpl_cpu_->set_pstate_speed(speed_per_state); });
+  kernel::actor::simcall_object_access(pimpl_,
+                                       [this, &speed_per_state] { pimpl_cpu_->set_pstate_speed(speed_per_state); });
   return this;
 }
 
@@ -300,7 +307,7 @@ std::vector<double> Host::convert_pstate_speed_vector(const std::vector<std::str
       double speed = xbt_parse_get_speed("", 0, speed_str, "");
       speed_list.push_back(speed);
     } catch (const simgrid::ParseError&) {
-      throw std::invalid_argument(std::string("Invalid speed value: ") + speed_str);
+      throw std::invalid_argument("Invalid speed value: " + speed_str);
     }
   }
   return speed_list;
@@ -315,7 +322,7 @@ Host* Host::set_pstate_speed(const std::vector<std::string>& speed_per_state)
 /** @brief Set the pstate at which the host should run */
 Host* Host::set_pstate(unsigned long pstate_index)
 {
-  kernel::actor::simcall([this, pstate_index] { this->pimpl_cpu_->set_pstate(pstate_index); });
+  kernel::actor::simcall_object_access(pimpl_, [this, pstate_index] { this->pimpl_cpu_->set_pstate(pstate_index); });
   return this;
 }
 
@@ -327,14 +334,14 @@ unsigned long Host::get_pstate() const
 
 Host* Host::set_factor_cb(const std::function<CpuFactorCb>& cb)
 {
-  kernel::actor::simcall([this, &cb] { pimpl_cpu_->set_factor_cb(cb); });
+  kernel::actor::simcall_object_access(pimpl_, [this, &cb] { pimpl_cpu_->set_factor_cb(cb); });
   return this;
 }
 
 Host* Host::set_coordinates(const std::string& coords)
 {
   if (not coords.empty())
-    kernel::actor::simcall([this, coords] { this->pimpl_netpoint_->set_coordinates(coords); });
+    kernel::actor::simcall_object_access(pimpl_, [this, coords] { this->pimpl_netpoint_->set_coordinates(coords); });
   return this;
 }
 std::vector<Disk*> Host::get_disks() const
@@ -342,9 +349,13 @@ std::vector<Disk*> Host::get_disks() const
   return this->pimpl_->get_disks();
 }
 
+Disk* Host::get_disk_by_name(const std::string& name) const
+{
+  return this->pimpl_->get_disk_by_name(name);
+}
 Disk* Host::create_disk(const std::string& name, double read_bandwidth, double write_bandwidth)
 {
-  return kernel::actor::simcall([this, &name, read_bandwidth, write_bandwidth] {
+  return kernel::actor::simcall_answered([this, &name, read_bandwidth, write_bandwidth] {
     auto* disk = pimpl_->create_disk(name, read_bandwidth, write_bandwidth);
     pimpl_->add_disk(disk);
     return disk;
@@ -357,27 +368,43 @@ Disk* Host::create_disk(const std::string& name, const std::string& read_bandwid
   try {
     d_read = xbt_parse_get_bandwidth("", 0, read_bandwidth, "");
   } catch (const simgrid::ParseError&) {
-    throw std::invalid_argument(std::string("Impossible to create disk: ") + name +
-                                std::string(". Invalid read bandwidth: ") + read_bandwidth);
+    throw std::invalid_argument("Impossible to create disk: " + name + ". Invalid read bandwidth: " + read_bandwidth);
   }
   double d_write;
   try {
     d_write = xbt_parse_get_bandwidth("", 0, write_bandwidth, "");
   } catch (const simgrid::ParseError&) {
-    throw std::invalid_argument(std::string("Impossible to create disk: ") + name +
-                                std::string(". Invalid write bandwidth: ") + write_bandwidth);
+    throw std::invalid_argument("Impossible to create disk: " + name + ". Invalid write bandwidth: " + write_bandwidth);
   }
   return create_disk(name, d_read, d_write);
 }
 
 void Host::add_disk(const Disk* disk)
 {
-  kernel::actor::simcall([this, disk] { this->pimpl_->add_disk(disk); });
+  kernel::actor::simcall_answered([this, disk] { this->pimpl_->add_disk(disk); });
 }
 
 void Host::remove_disk(const std::string& disk_name)
 {
-  kernel::actor::simcall([this, disk_name] { this->pimpl_->remove_disk(disk_name); });
+  kernel::actor::simcall_answered([this, disk_name] { this->pimpl_->remove_disk(disk_name); });
+}
+
+VirtualMachine* Host::create_vm(const std::string& name, int core_amount)
+{
+  return kernel::actor::simcall_answered(
+      [this, &name, core_amount] { return this->pimpl_->create_vm(name, core_amount); });
+}
+
+VirtualMachine* Host::create_vm(const std::string& name, int core_amount, size_t ramsize)
+{
+  return kernel::actor::simcall_answered(
+      [this, &name, core_amount, ramsize] { return this->pimpl_->create_vm(name, core_amount, ramsize); });
+}
+
+VirtualMachine* Host::vm_by_name_or_null(const std::string& name)
+{
+  simgrid::kernel::resource::VirtualMachineImpl* vm = this->pimpl_->get_vm_by_name_or_null(name);
+  return vm ? vm->get_iface() : nullptr;
 }
 
 ExecPtr Host::exec_init(double flops) const
@@ -397,12 +424,12 @@ void Host::execute(double flops) const
 
 void Host::execute(double flops, double priority) const
 {
-  this_actor::exec_init(flops)->set_priority(1 / priority)->vetoable_start()->wait();
+  Exec::init()->set_flops_amount(flops)->set_host(const_cast<Host*>(this))->set_priority(1 / priority)->wait();
 }
 
 Host* Host::seal()
 {
-  kernel::actor::simcall([this]() { this->pimpl_->seal(); });
+  kernel::actor::simcall_answered([this]() { this->pimpl_->seal(); });
   simgrid::s4u::Host::on_creation(*this); // notify the signal
   return this;
 }
@@ -449,19 +476,10 @@ sg_host_t sg_host_by_name(const char* name)
   return simgrid::s4u::Host::by_name_or_null(name);
 }
 
-xbt_dynar_t sg_hosts_as_dynar() // XBT_ATTRIB_DEPRECATED_v330
+/** @brief Retrieve a VM running on a given host from its name, or return NULL if no VM matches*/
+sg_vm_t sg_vm_by_name(sg_host_t host, const char* name)
 {
-  std::vector<simgrid::s4u::Host*> list = simgrid::s4u::Engine::get_instance()->get_all_hosts();
-
-  auto last = std::remove_if(begin(list), end(list), [](const simgrid::s4u::Host* host) {
-    return not host || not host->get_netpoint() || not host->get_netpoint()->is_host();
-  });
-  std::sort(begin(list), last,
-            [](const simgrid::s4u::Host* a, const simgrid::s4u::Host* b) { return a->get_name() < b->get_name(); });
-
-  xbt_dynar_t res = xbt_dynar_new(sizeof(sg_host_t), nullptr);
-  std::for_each(begin(list), last, [res](sg_host_t host) { xbt_dynar_push_as(res, sg_host_t, host); });
-  return res;
+  return host->vm_by_name_or_null(name);
 }
 
 // ========= Layering madness ==============*
@@ -469,20 +487,12 @@ xbt_dynar_t sg_hosts_as_dynar() // XBT_ATTRIB_DEPRECATED_v330
 // ========== User data Layer ==========
 void* sg_host_get_data(const_sg_host_t host)
 {
-  return host->get_data();
+  return host->get_data<void>();
 }
 void sg_host_set_data(sg_host_t host, void* userdata)
 {
   host->set_data(userdata);
 }
-void* sg_host_data(const_sg_host_t host) // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_host_get_data(host);
-}
-void sg_host_data_set(sg_host_t host, void* userdata) // XBT_ATTRIB_DEPRECATED_v330
-{
-  sg_host_set_data(host, userdata);
-}
 
 // ========= Disk related functions ============
 void sg_host_get_disks(const_sg_host_t host, unsigned int* disk_count, sg_disk_t** disks)
@@ -492,7 +502,10 @@ void sg_host_get_disks(const_sg_host_t host, unsigned int* disk_count, sg_disk_t
   *disks                      = xbt_new(sg_disk_t, list.size());
   std::copy(begin(list), end(list), *disks);
 }
-
+sg_disk_t sg_host_get_disk_by_name(const_sg_host_t host, const char* name)
+{
+  return host->get_disk_by_name(name);
+}
 // =========== user-level functions ===============
 // ================================================
 /** @brief Returns the total speed of a host */
@@ -501,11 +514,6 @@ double sg_host_get_speed(const_sg_host_t host)
   return host->get_speed();
 }
 
-double sg_host_speed(const_sg_host_t host) // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_host_get_speed(host);
-}
-
 /** @brief Return the speed of the processor (in flop/s) at a given pstate. See also @ref plugin_host_energy.
  *
  * @param  host host to test
@@ -575,7 +583,7 @@ void sg_host_turn_on(sg_host_t host)
  *
  * @brief Stop the host if it is on
  *
- * See also #MSG_host_is_on() to test the current state of the host and @ref plugin_host_energy
+ * See also #sg_host_is_on() to test the current state of the host and @ref plugin_host_energy
  * for more info on DVFS.
  */
 void sg_host_turn_off(sg_host_t host)
@@ -605,8 +613,8 @@ xbt_dict_t sg_host_get_properties(const_sg_host_t host)
 
   if (props == nullptr)
     return nullptr;
-  for (auto const& elm : *props) {
-    xbt_dict_set(as_dict, elm.first.c_str(), xbt_strdup(elm.second.c_str()));
+  for (auto const& [key, value] : *props) {
+    xbt_dict_set(as_dict, key.c_str(), xbt_strdup(value.c_str()));
   }
   return as_dict;
 }
@@ -675,42 +683,11 @@ double sg_host_get_route_bandwidth(const_sg_host_t from, const_sg_host_t to)
   return min_bandwidth;
 }
 
-void sg_host_route(const_sg_host_t from, const_sg_host_t to, xbt_dynar_t links) // XBT_ATTRIB_DEPRECATED_v330
-{
-  sg_host_get_route(from, to, links);
-}
-
-double sg_host_route_latency(const_sg_host_t from, const_sg_host_t to) // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_host_get_route_latency(from, to);
-}
-
-double sg_host_route_bandwidth(const_sg_host_t from, const_sg_host_t to) // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_host_get_route_bandwidth(from, to);
-}
-
 void sg_host_sendto(sg_host_t from, sg_host_t to, double byte_amount)
 {
   simgrid::s4u::Comm::sendto(from, to, byte_amount);
 }
 
-/** @brief Displays debugging information about a host */
-void sg_host_dump(const_sg_host_t host)
-{
-  XBT_INFO("Displaying host %s", host->get_cname());
-  XBT_INFO("  - speed: %.0f", host->get_speed());
-  XBT_INFO("  - available speed: %.2f", sg_host_get_available_speed(host));
-  const std::unordered_map<std::string, std::string>* props = host->get_properties();
-
-  if (not props->empty()) {
-    XBT_INFO("  - properties:");
-    for (auto const& elm : *props) {
-      XBT_INFO("    %s->%s", elm.first.c_str(), elm.second.c_str());
-    }
-  }
-}
-
 /** @brief Return the list of actors attached to a host.
  *
  * @param host a host
@@ -744,8 +721,3 @@ double sg_host_get_load(const_sg_host_t host)
 {
   return host->get_load();
 }
-
-double sg_host_load(const_sg_host_t host) // XBT_ATTRIB_DEPRECATED_v330
-{
-  return sg_host_get_load(host);
-}
