@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -23,11 +23,10 @@ void smpi_init_fortran_types()
     MPI_COMM_WORLD->add_f();
     MPI_BYTE->add_f(); // MPI_BYTE
     MPI_CHAR->add_f(); // MPI_CHARACTER
+    MPI_C_BOOL->add_f(); // MPI_LOGICAL
     if (sizeof(void*) == 8) {
-      MPI_C_BOOL->add_f(); // MPI_LOGICAL
       MPI_INT->add_f();    // MPI_INTEGER
     } else {
-      MPI_C_BOOL->add_f(); // MPI_LOGICAL
       MPI_LONG->add_f();   // MPI_INTEGER
     }
     MPI_INT8_T->add_f();    // MPI_INTEGER1
@@ -570,10 +569,7 @@ void mpi_op_commutative_(int* op, int* commute, int* ierr)
 void mpi_group_free_(int* group, int* ierr)
 {
   MPI_Group tmp = simgrid::smpi::Group::f2c(*group);
-  if(tmp != MPI_COMM_WORLD->group() && tmp != MPI_GROUP_EMPTY){
-    simgrid::smpi::Group::unref(tmp);
-  }
-  *ierr = MPI_SUCCESS;
+  *ierr         = MPI_Group_free(&tmp);
 }
 
 void mpi_group_size_(int* group, int* size, int* ierr)
