@@ -1,6 +1,6 @@
 /* selector for collective algorithms based on mpich decision logic */
 
-/* Copyright (c) 2009-2021. The SimGrid Team.
+/* Copyright (c) 2009-2024. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -58,8 +58,7 @@
 
    End Algorithm: MPI_Allreduce
 */
-namespace simgrid{
-namespace smpi{
+namespace simgrid::smpi {
 int allreduce__mpich(const void *sbuf, void *rbuf, int count,
                         MPI_Datatype dtype, MPI_Op op, MPI_Comm comm)
 {
@@ -704,6 +703,4 @@ int scatter__mpich(const void *sbuf, int scount,
   }
   return scatter__ompi_binomial(sbuf, scount, sdtype, rbuf, rcount, rdtype, root, comm);
 }
-}
-}
-
+} // namespace simgrid::smpi
