@@ -1,6 +1,6 @@
 /* Asynchronous parts of the basic collective algorithms, meant to be used both for the naive default implementation, but also for non blocking collectives */
 
-/* Copyright (c) 2009-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2009-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -8,8 +8,7 @@
 #include "colls_private.hpp"
 #include "src/smpi/include/smpi_actor.hpp"
 
-namespace simgrid{
-namespace smpi{
+namespace simgrid::smpi {
 
 int colls::ibarrier(MPI_Comm comm, MPI_Request* request, int external)
 {
@@ -79,8 +78,8 @@ int colls::iallgather(const void* sendbuf, int sendcount, MPI_Datatype sendtype,
   for (int other = 0; other < size; other++) {
     if(other != rank) {
       requests.push_back(Request::isend_init(sendbuf, sendcount, sendtype, other, system_tag, comm));
-      requests.push_back(Request::irecv_init(static_cast<char *>(recvbuf) + other * recvcount * recvext, 
-                         recvcount, recvtype, other, system_tag, comm));
+      requests.push_back(Request::irecv_init(static_cast<char*>(recvbuf) + other * recvcount * recvext, recvcount,
+                                             recvtype, other, system_tag, comm));
     }
   }
   (*request)->start_nbc_requests(requests);
@@ -550,5 +549,4 @@ int colls::ireduce_scatter(const void* sendbuf, void* recvbuf, const int* recvco
   return MPI_SUCCESS;
 }
 
-}
-}
+} // namespace simgrid::smpi
