@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -8,13 +8,12 @@
 #include "src/smpi/include/smpi_actor.hpp"
 #include "src/instr/instr_smpi.hpp"
 
-int mpi_in_place_;
-int mpi_bottom_;
-int mpi_status_ignore_;
-int mpi_statuses_ignore_;
+const int mpi_in_place_        = -222;
+const int mpi_bottom_          = -111;
+const int mpi_status_ignore_   = 0;
+const int mpi_statuses_ignore_ = 0;
 
-namespace simgrid{
-namespace smpi{
+namespace simgrid::smpi {
 
 std::unique_ptr<F2C::f2c_lookup_type> F2C::f2c_lookup_    = nullptr;
 int F2C::f2c_id_ = 0;
@@ -26,8 +25,7 @@ F2C::F2C() = default;
 int F2C::add_f()
 {
   allocate_lookup();
-  auto loc = smpi_process()->call_location();
-  if(loc && loc->linenumber != 0)
+  if (auto const* loc = smpi_process()->call_location(); loc && loc->linenumber != 0)
     call_location_= std::string (loc->filename + ":" + std::to_string(loc->linenumber));
   my_f2c_id_                 = global_f2c_id();
   (*f2c_lookup_)[my_f2c_id_] = this;
@@ -57,5 +55,4 @@ F2C* F2C::f2c(int id)
     return nullptr;
 }
 
-}
-}
+} // namespace simgrid::smpi
