@@ -1,10 +1,9 @@
-/* Copyright (c) 2005-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2005-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "simgrid/Exception.hpp"
-#include "src/kernel/context/Context.hpp"
 #include <xbt/config.hpp>
 #include <xbt/log.hpp>
 
@@ -18,15 +17,14 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(xbt_exception, xbt, "Exceptions");
 
 void _xbt_throw(char* message, const char* file, int line, const char* func)
 {
-  simgrid::Exception e(
-      simgrid::xbt::ThrowPoint(file, line, func, simgrid::xbt::Backtrace(), xbt_procname(), xbt_getpid()),
-      message ? message : "");
+  simgrid::Exception e(simgrid::xbt::ThrowPoint(file, line, func, simgrid::xbt::Backtrace(), sg_actor_self_get_name(),
+                                                sg_actor_self_get_pid()),
+                       message ? message : "");
   xbt_free(message);
   throw e;
 }
 
-namespace simgrid {
-namespace xbt {
+namespace simgrid::xbt {
 
 ImpossibleError::~ImpossibleError()         = default;
 InitializationError::~InitializationError() = default;
@@ -37,9 +35,9 @@ void log_exception(e_xbt_log_priority_t prio, const char* context, std::exceptio
   try {
     std::string name = boost::core::demangle(typeid(exception).name());
 
-    auto* with_context = dynamic_cast<const simgrid::Exception*>(&exception);
+    const auto* with_context = dynamic_cast<const simgrid::Exception*>(&exception);
     if (with_context != nullptr) {
-      XBT_LOG(prio, "%s %s by %s/%d: %s", context, name.c_str(), with_context->throw_point().procname_.c_str(),
+      XBT_LOG(prio, "%s %s by %s/%ld: %s", context, name.c_str(), with_context->throw_point().procname_.c_str(),
               with_context->throw_point().pid_, exception.what());
       // Do we have a backtrace?
       if (not simgrid::config::get_value<bool>("exception/cutpath")) {
@@ -56,7 +54,7 @@ void log_exception(e_xbt_log_priority_t prio, const char* context, std::exceptio
 
   try {
     // Do we have a nested exception?
-    auto* with_nested = dynamic_cast<const std::nested_exception*>(&exception);
+    const auto* with_nested = dynamic_cast<const std::nested_exception*>(&exception);
     if (with_nested != nullptr && with_nested->nested_ptr() != nullptr)
       with_nested->rethrow_nested();
   } catch (const std::exception& nested_exception) {
@@ -82,8 +80,7 @@ static std::terminate_handler previous_terminate_handler = nullptr;
 XBT_ATTRIB_NORETURN static void handler()
 {
   // Avoid doing crazy things if we get an uncaught exception inside an uncaught exception
-  static std::atomic_flag lock = ATOMIC_FLAG_INIT;
-  if (lock.test_and_set()) {
+  if (static std::atomic_flag lock = ATOMIC_FLAG_INIT; lock.test_and_set()) {
     XBT_ERROR("Handling an exception raised an exception. Bailing out.");
     std::abort();
   }
@@ -106,8 +103,7 @@ XBT_ATTRIB_NORETURN static void handler()
   }
 
   catch (const simgrid::ForcefulKillException&) {
-    XBT_ERROR("Received a ForcefulKillException at the top-level exception handler. Maybe a Java->C++ call that is not "
-              "protected in a try/catch?");
+    XBT_ERROR("Received a ForcefulKillException at the top-level exception handler. Please help fix this bug.");
   }
 
   // We don't know how to manage other exceptions
@@ -131,8 +127,7 @@ void install_exception_handler()
   });
 }
 
-} // namespace xbt
-} // namespace simgrid
+} // namespace simgrid::xbt
 
 void xbt_throw_impossible(const char* file, int line, const char* func)
 {
