@@ -1,6 +1,6 @@
 /* xbt_os_time.c -- portable interface to time-related functions            */
 
-/* Copyright (c) 2007-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2007-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -13,12 +13,7 @@
 #include <sys/time.h>
 #include <time.h>
 
-#ifdef _WIN32
-#include <sys/timeb.h>
-#include <windows.h>
-#else
 #include <unistd.h>
-#endif
 
 //Freebsd doesn't provide this clock_gettime flag yet, because it was added too recently (after 1993)
 #if defined (CLOCK_PROF) && ! defined (CLOCK_PROCESS_CPUTIME_ID)
@@ -40,56 +35,22 @@
 #include <mach/task.h>
 #endif
 
-#ifdef _WIN32
-static void w32_time_to_timeval(struct timeval* tv, const FILETIME* ft)
-{
-  unsigned __int64 tm;
-  tm = (unsigned __int64)ft->dwHighDateTime << 32;
-  tm |= ft->dwLowDateTime;
-  tm /= 10;
-  tm -= 11644473600000000ULL;
-  tv->tv_sec  = (long)(tm / 1000000L);
-  tv->tv_usec = (long)(tm % 1000000L);
-}
-
-static void w32_times_to_timeval(struct timeval* tv, const FILETIME* kernel_time, const FILETIME* user_time)
-{
-  unsigned __int64 ktm, utm;
-  ktm = (unsigned __int64)kernel_time->dwHighDateTime << 32;
-  ktm |= kernel_time->dwLowDateTime;
-  ktm /= 10;
-  utm = (unsigned __int64)user_time->dwHighDateTime << 32;
-  utm |= user_time->dwLowDateTime;
-  utm /= 10;
-  tv->tv_sec  = (long)(ktm / 1000000L) + (long)(utm / 1000000L);
-  tv->tv_usec = (long)(ktm % 1000000L) + (long)(utm % 1000000L);
-}
-#endif
-
 double xbt_os_time(void)
 {
 #if HAVE_GETTIMEOFDAY
   struct timeval tv;
   gettimeofday(&tv, NULL);
-#elif defined(_WIN32)
-  struct timeval tv;
-  FILETIME ft;
-  GetSystemTimeAsFileTime(&ft);
-  w32_time_to_timeval(&tv, &ft);
-#else                           /* not windows, no gettimeofday => poor resolution */
-  return (double) (time(NULL));
-#endif                          /* HAVE_GETTIMEOFDAY? */
 
   return (double)tv.tv_sec + (double)tv.tv_usec / 1e6;
+#else                           /* no gettimeofday => poor resolution */
+  return (double) (time(NULL));
+#endif                          /* HAVE_GETTIMEOFDAY? */
 }
 
 void xbt_os_sleep(double sec)
 {
 
-#ifdef _WIN32
-  Sleep((floor(sec) * 1000) + ((sec - floor(sec)) * 1000));
-
-#elif HAVE_NANOSLEEP
+#if HAVE_NANOSLEEP
   struct timespec ts;
   ts.tv_sec  = (time_t)sec;
   ts.tv_nsec = (long)((sec - floor(sec)) * 1e9);
@@ -98,7 +59,7 @@ void xbt_os_sleep(double sec)
   struct timeval timeout;
 
   timeout.tv_sec  = (long)sec;
-  timeout.tv_usec = (long)(sec - floor(sec)) * 1e6);
+  timeout.tv_usec = (long)(sec - floor(sec)) * 1e6;
 
   select(0, NULL, NULL, NULL, &timeout);
 #endif
@@ -114,7 +75,7 @@ struct s_xbt_os_timer {
   struct timespec start;
   struct timespec stop;
   struct timespec elapse;
-#elif HAVE_GETTIMEOFDAY || defined(_WIN32)
+#elif HAVE_GETTIMEOFDAY
   struct timeval start;
   struct timeval stop;
   struct timeval elapse;
@@ -145,7 +106,7 @@ double xbt_os_timer_elapsed(const_xbt_os_timer_t timer)
 #if HAVE_POSIX_GETTIME && defined (_POSIX_THREAD_CPUTIME)
   return ((double) timer->stop.tv_sec) - ((double) timer->start.tv_sec) + ((double) timer->elapse.tv_sec ) +
       ((((double) timer->stop.tv_nsec) - ((double) timer->start.tv_nsec) + ((double) timer->elapse.tv_nsec )) / 1e9);
-#elif HAVE_GETTIMEOFDAY || defined(_WIN32)
+#elif HAVE_GETTIMEOFDAY
   return ((double) timer->stop.tv_sec) - ((double) timer->start.tv_sec) + ((double) timer->elapse.tv_sec ) +
       ((((double) timer->stop.tv_usec) - ((double) timer->start.tv_usec) + ((double) timer->elapse.tv_usec )) /
          1000000.0);
@@ -164,12 +125,6 @@ void xbt_os_walltimer_start(xbt_os_timer_t timer)
   timer->elapse.tv_sec = 0;
   timer->elapse.tv_usec = 0;
   gettimeofday(&(timer->start), NULL);
-#elif defined(_WIN32)
-  timer->elapse.tv_sec = 0;
-  timer->elapse.tv_usec = 0;
-  FILETIME ft;
-  GetSystemTimeAsFileTime(&ft);
-  w32_time_to_timeval(&timer->start, &ft);
 #else
   timer->elapse = 0;
   timer->start = (unsigned long int) (time(NULL));
@@ -187,12 +142,6 @@ void xbt_os_walltimer_resume(xbt_os_timer_t timer)
   timer->elapse.tv_sec += timer->stop.tv_sec - timer->start.tv_sec;
   timer->elapse.tv_usec += timer->stop.tv_usec - timer->start.tv_usec;
   gettimeofday(&(timer->start), NULL);
-#elif defined(_WIN32)
-  timer->elapse.tv_sec += timer->stop.tv_sec - timer->start.tv_sec;
-  timer->elapse.tv_usec += timer->stop.tv_usec - timer->start.tv_usec;
-  FILETIME ft;
-  GetSystemTimeAsFileTime(&ft);
-  w32_time_to_timeval(&timer->start, &ft);
 #else
   timer->elapse = timer->stop - timer->start;
   timer->start = (unsigned long int) (time(NULL));
@@ -205,10 +154,6 @@ void xbt_os_walltimer_stop(xbt_os_timer_t timer)
   clock_gettime(CLOCK_REALTIME, &(timer->stop));
 #elif HAVE_GETTIMEOFDAY
   gettimeofday(&(timer->stop), NULL);
-#elif defined(_WIN32)
-  FILETIME ft;
-  GetSystemTimeAsFileTime(&ft);
-  w32_time_to_timeval(&timer->stop, &ft);
 #else
   timer->stop = (unsigned long int) (time(NULL));
 #endif
@@ -224,13 +169,6 @@ void xbt_os_cputimer_start(xbt_os_timer_t timer)
   timer->elapse.tv_sec = 0;
   timer->elapse.tv_usec = 0;
   gettimeofday(&(timer->start), NULL);
-#elif defined(_WIN32)
-  timer->elapse.tv_sec = 0;
-  timer->elapse.tv_usec = 0;
-  HANDLE h = GetCurrentProcess();
-  FILETIME creationTime, exitTime, kernelTime, userTime;
-  GetProcessTimes(h, &creationTime, &exitTime, &kernelTime, &userTime);
-  w32_times_to_timeval(&timer->start, &kernelTime, &userTime);
 #else
 # error The cpu timers of SimGrid do not seem to work on your platform.
 #endif
@@ -246,13 +184,6 @@ void xbt_os_cputimer_resume(xbt_os_timer_t timer)
   timer->elapse.tv_sec += timer->stop.tv_sec - timer->start.tv_sec;
   timer->elapse.tv_usec += timer->stop.tv_usec - timer->start.tv_usec;
   gettimeofday(&(timer->start), NULL);
-#elif defined(_WIN32)
-  timer->elapse.tv_sec += timer->stop.tv_sec - timer->start.tv_sec;
-  timer->elapse.tv_usec += timer->stop.tv_usec - timer->start.tv_usec;
-  HANDLE h = GetCurrentProcess();
-  FILETIME creationTime, exitTime, kernelTime, userTime;
-  GetProcessTimes(h, &creationTime, &exitTime, &kernelTime, &userTime);
-  w32_times_to_timeval(&timer->start, &kernelTime, &userTime);
 #else
 # error The cpu timers of SimGrid do not seem to work on your platform.
 #endif
@@ -264,11 +195,6 @@ void xbt_os_cputimer_stop(xbt_os_timer_t timer)
   clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &(timer->stop));
 #elif HAVE_GETTIMEOFDAY
   gettimeofday(&(timer->stop), NULL);
-#elif defined(_WIN32)
-  HANDLE h = GetCurrentProcess();
-  FILETIME creationTime, exitTime, kernelTime, userTime;
-  GetProcessTimes(h, &creationTime, &exitTime, &kernelTime, &userTime);
-  w32_times_to_timeval(&timer->stop, &kernelTime, &userTime);
 #else
 # error The cpu timers of SimGrid do not seem to work on your platform.
 #endif
@@ -293,11 +219,6 @@ void xbt_os_threadtimer_start(xbt_os_timer_t timer)
   timer->elapse.tv_sec = 0;
   timer->elapse.tv_usec = 0;
   gettimeofday(&(timer->start), NULL);
-#elif defined(_WIN32)
-  HANDLE h = GetCurrentThread();
-  FILETIME creationTime, exitTime, kernelTime, userTime;
-  GetThreadTimes(h, &creationTime, &exitTime, &kernelTime, &userTime);
-  w32_times_to_timeval(&timer->start, &kernelTime, &userTime);
 #else
 # error The thread timers of SimGrid do not seem to work on your platform.
 #endif
@@ -322,13 +243,6 @@ void xbt_os_threadtimer_resume(xbt_os_timer_t timer)
   timer->elapse.tv_sec += timer->stop.tv_sec - timer->start.tv_sec;
   timer->elapse.tv_usec += timer->stop.tv_usec - timer->start.tv_usec;
   gettimeofday(&(timer->start), NULL);
-#elif defined(_WIN32)
-  timer->elapse.tv_sec += timer->stop.tv_sec - timer->start.tv_sec;
-  timer->elapse.tv_usec += timer->stop.tv_usec - timer->start.tv_usec;
-  HANDLE h = GetCurrentThread();
-  FILETIME creationTime, exitTime, kernelTime, userTime;
-  GetThreadTimes(h, &creationTime, &exitTime, &kernelTime, &userTime);
-  w32_times_to_timeval(&timer->start, &kernelTime, &userTime);
 #else
 # error The thread timers of SimGrid do not seem to work on your platform.
 #endif
@@ -347,11 +261,6 @@ void xbt_os_threadtimer_stop(xbt_os_timer_t timer)
   timer->stop.tv_sec = thi->system_time.seconds + thi->user_time.seconds;
 #elif HAVE_GETTIMEOFDAY //if nothing else is available, return just time
   gettimeofday(&(timer->stop), NULL);
-#elif defined(_WIN32)
-  HANDLE h = GetCurrentThread();
-  FILETIME creationTime, exitTime, kernelTime, userTime;
-  GetThreadTimes(h, &creationTime, &exitTime, &kernelTime, &userTime);
-  w32_times_to_timeval(&timer->stop, &kernelTime, &userTime);
 #else
 # error The thread timers of SimGrid do not seem to work on your platform.
 #endif
