@@ -1,4 +1,4 @@
-/* Copyright (c) 2020-2021. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2020-2024. The SimGrid Team. All rights reserved.          */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -14,21 +14,23 @@
 
 XBT_LOG_NEW_DEFAULT_CATEGORY(mwe, "Minimum Working Example");
 
-simgrid::s4u::ActorPtr receiver;
-
 class Receiver {
 public:
   void operator()() const
   {
     XBT_INFO("Starting.");
-    auto mailbox = simgrid::s4u::Mailbox::by_name("receiver");
+    auto* mailbox = simgrid::s4u::Mailbox::by_name("receiver");
     int data     = *mailbox->get<int>();
     XBT_INFO("Got %d at the end", data);
   }
 };
 
 class Suspender {
+  const simgrid::s4u::ActorPtr& receiver;
+
 public:
+  explicit Suspender(const simgrid::s4u::ActorPtr& receiver) : receiver(receiver) {}
+
   void operator()() const
   {
     XBT_INFO("Suspend the receiver...");
@@ -40,7 +42,7 @@ public:
     simgrid::s4u::this_actor::sleep_for(10);
 
     XBT_INFO("Sending a message to the receiver...");
-    auto mailbox = simgrid::s4u::Mailbox::by_name("receiver");
+    auto* mailbox   = simgrid::s4u::Mailbox::by_name("receiver");
     static int data = 42;
     mailbox->put(&data, 4);
 
@@ -55,7 +57,8 @@ int main(int argc, char** argv)
   engine.load_platform(argv[1]);
   simgrid::s4u::Host* host = engine.host_by_name("Tremblay");
 
-  simgrid::s4u::Actor::create("Suspender", host, Suspender());
+  simgrid::s4u::ActorPtr receiver;
+  simgrid::s4u::Actor::create("Suspender", host, Suspender(receiver));
   receiver = simgrid::s4u::Actor::create("Receiver", host, Receiver());
 
   engine.run();
