@@ -1,4 +1,4 @@
-/* Copyright (c) 2008-2021. The SimGrid Team.
+/* Copyright (c) 2008-2024. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -29,7 +29,7 @@ int main(int argc, char** argv)
   for (const auto& l : links) {
     XBT_INFO("%s: latency = %.5f, bandwidth = %f", l->get_cname(), l->get_latency(), l->get_bandwidth());
     l->set_data(&user_data);
-    xbt_assert(user_data == *static_cast<const std::string*>(l->get_data()), "User data was corrupted.");
+    xbt_assert(user_data == *l->get_data<std::string>(), "User data was corrupted.");
   }
 
   return 0;
