@@ -1,6 +1,6 @@
 #!/usr/bin/env perl
 
-# Copyright (c) 2012-2021. The SimGrid Team. All rights reserved.
+# Copyright (c) 2012-2024. The SimGrid Team. All rights reserved.
 
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the license (GNU LGPL) which comes with this package.
@@ -8,7 +8,8 @@
 use strict;
 use warnings;
 
-my @argv = ("valgrind");
+# Many other parameters (such as trace-children) are set in Tests.cmake
+my @argv = ("valgrind", "--quiet");
 my $count = 0;
 
 while (defined(my $arg = shift)) {
