@@ -2,10 +2,10 @@
 eval 'exec perl -S $0 ${1+"$@"}'
     if $running_under_some_shell;
 
-# This script updates the simgrid XML file passed as argument (modification in place)
+# This script updates the SimGrid XML file passed as argument (modification in place)
 # It is built to do the conversion incrementally.
 
-# Copyright (c) 2006-2021. The SimGrid Team.
+# Copyright (c) 2006-2024. The SimGrid Team.
 # All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
@@ -15,7 +15,7 @@ eval 'exec perl -S $0 ${1+"$@"}'
 
 =head1 NAME
 
-simgrid_update_xml - updates simgrid XML files to latest version
+simgrid_update_xml - updates SimGrid XML files to latest version
 
 =head1 SYNOPSIS
 
@@ -23,7 +23,7 @@ B<simgrid_update_xml> I<xml_file>
 
 =head1 DESCRIPTION
 
-simgrid_update_xml updates the simgrid XML file passed as argument.  The file
+simgrid_update_xml updates the SimGrid XML file passed as argument.  The file
 is modified in place, without any kind of backup. You may want to save a copy
 before running the script.
 
@@ -156,11 +156,11 @@ In <host> and <peer>, rename the 'availability_file' attribute into 'speed_file'
 
 =head1 AUTHORS
 
- The SimGrid team (simgrid-devel@lists.gforge.inria.fr)
+ The SimGrid team
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006-2021. The SimGrid Team. All rights reserved.
+Copyright (c) 2006-2024. The SimGrid Team. All rights reserved.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GNU LGPL (v2.1) license.
@@ -247,11 +247,11 @@ while (defined($line = <INPUT>)) {
 	    $output_string .=  " <AS  id=\"AS0\"  routing=\"Full\">\n";
 	    $AS_opened=1;
 	}
-	
+
 	if($line=~/<route /){$line =~ s/\<route/\<route symmetrical=\"NO\"/g;}
     }
     if ($fromversion < 4) {
-	$line =~ s/\bpower\b/speed/g;	
+	$line =~ s/\bpower\b/speed/g;
 	$line =~ s/\bkind="POWER"/kind="SPEED"/g;
     }
     if ($fromversion < 4.1) {
@@ -262,7 +262,7 @@ while (defined($line = <INPUT>)) {
     }
     $line =~ s/\bFULLDUPLEX\b/SPLITDUPLEX/g;
     $line =~ s/\bavailability_file\b/speed_file/g;
-	
+
     $output_string .= "$line\n";
 }
 
